GLR_Mail = {}

local L = LibStub("AceLocale-3.0"):GetLocale("GuildLotteryRaffle")
local DoHook = LibStub("AceHook-3.0")
local GLR_Mail_Chat = DEFAULT_CHAT_FRAME
local MAX_MAIL_SHOWN = 50
local mailIndex
local numUnshownItems
local lastNumGold
local wait
local button
local Postal_OpenAllMenuButton
local firstMailDaysLeft

-- Frame to refresh the Inbox
-- I'm cheap so instead of trying to track 60 or so seconds since the
-- last CheckInbox(), I just call CheckInbox() every 10 seconds
local GLR_RefreshFrame = CreateFrame("Frame", nil, MailFrame)
GLR_RefreshFrame:Hide()
GLR_RefreshFrame:SetScript("OnShow", function(self)
	self.time = 10
	self.mode = nil
end)
GLR_RefreshFrame:SetScript("OnUpdate", function(self, elapsed)
	self.time = self.time - elapsed
	if self.time <= 0 then
		if self.mode == nil then
			self.time = 10
			GLR:Print(L["Refreshing mailbox..."])
			GLR:RegisterEvent("MAIL_INBOX_UPDATE")
			CheckInbox()
			GLR_RefreshFrame:OnEvent()
		else
			self:Hide()
			GLR_Mail:OpenAll(true)
		end
	end
end)

local SubjectPatterns = {
	AHCancelled = gsub(AUCTION_REMOVED_MAIL_SUBJECT, "%%s", ".*"),
	AHExpired = gsub(AUCTION_EXPIRED_MAIL_SUBJECT, "%%s", ".*"),
	AHOutbid = gsub(AUCTION_OUTBID_MAIL_SUBJECT, "%%s", ".*"),
	AHSuccess = gsub(AUCTION_SOLD_MAIL_SUBJECT, "%%s", ".*"),
	AHWon = gsub(AUCTION_WON_MAIL_SUBJECT, "%%s", ".*"),
}

--function GLR_Mail:GetMailType(subject)
local function GetMailType(subject)
	if subject then
		for k, v in pairs(SubjectPatterns) do
			if subject:find(v) then return k end
		end
	end
	return "NonAHMail"
end

local function CountMoney()
	local numGold = 0
	for i = 1, GetInboxNumItems() do
		local _, _, sender, subject, money, CODAmount, _, hasItem, _, _, msgText, _, isGM = GetInboxHeaderInfo(i)
		numGold = numGold + money
	end
	return numGold
end

local function GetMoneyString(money)
	local gold = floor(money / 10000)
	if gold > 0 then
		return gold
	else
		gold = 0
		return gold
	end
end

local function Reset()
	
	GLR_RefreshFrame:Hide()
	GLR_Mail.GLR_UpdateFrame:Hide()
	GLR:UnregisterEvent("UI_ERROR_MESSAGE")
	
	GLR:Print(L["Mail Scan Complete"])
	GLR_UI.GLR_ScanMailButtonText:SetText(L["Scan Mail Button"])
	
	GLR_Mail:DisableInbox()
	InboxFrame_Update()
	if event == "MAIL_CLOSED" or event == "PLAYER_LEAVING_WORLD" then
		GLR:UnregisterEvent("MAIL_CLOSED")
		GLR:UnregisterEvent("PLAYER_LEAVING_WORLD")
	end
end

local function MailSetTickets(sender, numTickets, raffle)
	
	local faction, _ = UnitFactionGroup("PLAYER")
	local MultiGuild = glrCharacter.Data.Settings.MultiGuild
	local Status = raffle
	local CurrentNumTickets = 0
	local ExtraGold = 0
	local ExtraTickets = 0
	local overLimit = 0
	local progress = true
	local continue = false
	local guaranteedWinner = false
	local ChatInfo = glrCharacter.Data.Settings.ToggleChatInfo
	local numTotalMembers, _, _ = GetNumGuildMembers()
	local dateTime = time()
	local state = "Lottery"
	
	if status == 1 then
		state = "Raffle"
	end
	
	if not MultiGuild then
		for i = 1, numTotalMembers do
			local name, _, _, _, _, _, _, _, isOnline, _, _, _, _, _, _, _ = GetGuildRosterInfo(i)
			if sender == name then
				if isOnline then
					continue = true
				end
				break
			end
		end
	end
	
	if glrTemp == nil then
		glrTemp = {}
	end
	
	if state == "Lottery" then
		if glrCharacter.Event[state].SecondPlace == L["Winner Guaranteed"] then
			guaranteedWinner = true
		end
	else
		guaranteedWinner = true
	end
	
	local maxTickets = tonumber(glrCharacter.Event[state].MaxTickets)
	local TicketCost = tonumber(glrCharacter.Event[state].TicketPrice)
	
	if numTickets >= maxTickets then
		local ct = 0
		if glrCharacter[state].Entries.Tickets ~= nil then
			if glrCharacter[state].Entries.Tickets[sender] ~= nil then
				if glrCharacter[state].Entries.Tickets[sender].NumberOfTickets ~= nil then
					ct = glrCharacter[state].Entries.Tickets[sender].NumberOfTickets
					if maxTickets == ct then
						ExtraGold = numTickets * TicketCost
						ExtraTickets = numTickets
						numTickets = 0
					else
						local td = maxTickets - ct
						if numTickets > td then
							ExtraTickets = numTickets - maxTickets
							ExtraGold = ExtraTickets * TicketCost
							numTickets = td
						end
					end
				else
					if numTickets > maxTickets then
						ExtraTickets = numTickets - maxTickets
						ExtraGold = ExtraTickets * TicketCost
						numTickets = maxTickets
					end
				end
			else
				if numTickets > maxTickets then
					ExtraTickets = numTickets - maxTickets
					ExtraGold = ExtraTickets * TicketCost
					numTickets = maxTickets
				end
			end
		end
	end
	
	if ExtraTickets > 0 and ChatInfo then
		GLR:Print(sender .. " " .. L["Mail Scan Extra Ticket String 1"] .. " " .. ExtraGold .. " " .. L["Mail Scan Extra Ticket String 2"] .. ". " .. L["Mail Scan Extra Ticket String 3"] .. ".")
	end
	
	if glrTemp[state] == nil then
		glrTemp[state] = {}
	end
	if glrTemp[state][sender] == nil then
		glrTemp[state][sender] = {}
	end
	if glrTemp[state][sender].TicketNumbers == nil then
		glrTemp[state][sender].TicketNumbers = {}
	end
	if glrTemp[state][sender].NumberOfTickets == nil then
		glrTemp[state][sender].NumberOfTickets = {}
	end
	if glrCharacter[state].Entries == nil then
		glrCharacter[state].Entries = {}
	end
	if glrCharacter[state].Entries.TicketRange == nil then
		glrCharacter[state].Entries.TicketRange = {}
	end
	if glrCharacter[state].Entries.TicketRange[sender] == nil then
		glrCharacter[state].Entries.TicketRange[sender] = { ["LowestTicketNumber"] = 0, ["HighestTicketNumber"] = 0 }
	end
	if glrTemp[state][sender].TicketRange == nil then
		glrTemp[state][sender].TicketRange = { ["LowestTicketNumber"] = 0, ["HighestTicketNumber"] = 0 }
	end
	
	if glrCharacter[state].Entries.Tickets[sender] == nil then
		
		glrCharacter[state].Entries.Tickets[sender] = {}
		glrCharacter[state].Entries.Tickets[sender].NumberOfTickets = {}
		glrCharacter[state].Entries.Tickets[sender].NumberOfTickets = numTickets
		glrCharacter.Event[state].TicketsSold = glrCharacter.Event[state].TicketsSold + numTickets
		
		table.insert(glrCharacter[state].Entries.Names, sender)
		
		local TicketNumber = glrCharacter.Event[state].LastTicketBought
		local FirstTicketNumberBought = true
		local LastTicketNumberBought = true
		
		glrCharacter[state].Entries.TicketNumbers[sender] = {}
		
		if glrCharacter[state].TicketPool[1] ~= nil then
			
			for i = 1, numTickets do
				
				if glrCharacter[state].TicketPool[1] ~= nil then
					
					table.insert(glrCharacter[state].Entries.TicketNumbers[sender], glrCharacter[state].TicketPool[1])
					table.insert(glrTemp[state][sender].TicketNumbers, glrCharacter[state].TicketPool[1])
					if FirstTicketNumberBought then
						glrCharacter[state].Entries.TicketRange[sender].LowestTicketNumber = tonumber(glrCharacter[state].TicketPool[1])
						glrTemp[state][sender].TicketRange.LowestTicketNumber = tonumber(glrCharacter[state].TicketPool[1])
						FirstTicketNumberBought = false
					end
					if numTickets == i and LastTicketNumberBought then
						glrCharacter[state].Entries.TicketRange[sender].HighestTicketNumber = tonumber(glrCharacter[state].TicketPool[1])
						glrTemp[state][sender].TicketRange.HighestTicketNumber = tonumber(glrCharacter[state].TicketPool[1])
						LastTicketNumberBought = false
					end
					table.remove(glrCharacter[state].TicketPool, 1)
					
				else
					
					if TicketNumber == 1000 or TicketNumber == 2000 or TicketNumber == 3000 or TicketNumber == 4000 or TicketNumber == 5000 or TicketNumber == 6000 or TicketNumber == 7000 or TicketNumber == 8000 or TicketNumber == 9000 then
						local TicketString = tostring(TicketNumber)
						table.insert(glrCharacter[state].Entries.TicketNumbers[sender], TicketString)
						table.insert(glrTemp[state][sender].TicketNumbers, TicketString)
						if FirstTicketNumberBought then
							glrCharacter[state].Entries.TicketRange[sender].LowestTicketNumber = TicketNumber
							glrTemp[state][sender].TicketRange.LowestTicketNumber = TicketNumber
							FirstTicketNumberBought = false
						end
						if numTickets == i and LastTicketNumberBought then
							glrCharacter[state].Entries.TicketRange[sender].HighestTicketNumber = TicketNumber
							glrTemp[state][sender].TicketRange.HighestTicketNumber = TicketNumber
							LastTicketNumberBought = false
						end
						if guaranteedWinner then
							TicketNumber = TicketNumber + 1
						else
							TicketNumber = TicketNumber + 2
						end
						glrCharacter.Event[state].LastTicketBought = TicketNumber
					else
						local TicketString = tostring(TicketNumber)
						table.insert(glrCharacter[state].Entries.TicketNumbers[sender], TicketString)
						table.insert(glrTemp[state][sender].TicketNumbers, TicketString)
						if FirstTicketNumberBought then
							glrCharacter[state].Entries.TicketRange[sender].LowestTicketNumber = TicketNumber
							glrTemp[state][sender].TicketRange.LowestTicketNumber = TicketNumber
							FirstTicketNumberBought = false
						end
						if numTickets == i and LastTicketNumberBought then
							glrCharacter[state].Entries.TicketRange[sender].HighestTicketNumber = TicketNumber
							glrTemp[state][sender].TicketRange.HighestTicketNumber = TicketNumber
							LastTicketNumberBought = false
						end
						if guaranteedWinner then
							glrCharacter.Event[state].LastTicketBought = TicketNumber + 1
							TicketNumber = TicketNumber + 1
						else
							glrCharacter.Event[state].LastTicketBought = TicketNumber + 2
							TicketNumber = TicketNumber + 2
						end
					end
					
				end
				
			end
			
			if glrCharacter[state].TicketPool[1] ~= nil then
				sort(glrCharacter[state].TicketPool)
			end
			
		else
			
			for i = 1, numTickets do
				
				if TicketNumber == 1000 or TicketNumber == 2000 or TicketNumber == 3000 or TicketNumber == 4000 or TicketNumber == 5000 or TicketNumber == 6000 or TicketNumber == 7000 or TicketNumber == 8000 or TicketNumber == 9000 then
					local TicketString = tostring(TicketNumber)
					table.insert(glrCharacter[state].Entries.TicketNumbers[sender], TicketString)
					table.insert(glrTemp[state][sender].TicketNumbers, TicketString)
					if FirstTicketNumberBought then
						glrCharacter[state].Entries.TicketRange[sender].LowestTicketNumber = TicketNumber
						glrTemp[state][sender].TicketRange.LowestTicketNumber = TicketNumber
						FirstTicketNumberBought = false
					end
					if numTickets == i and LastTicketNumberBought then
						glrCharacter[state].Entries.TicketRange[sender].HighestTicketNumber = TicketNumber
						glrTemp[state][sender].TicketRange.HighestTicketNumber = TicketNumber
						LastTicketNumberBought = false
					end
					if guaranteedWinner then
						TicketNumber = TicketNumber + 1
					else
						TicketNumber = TicketNumber + 2
					end
					glrCharacter.Event[state].LastTicketBought = TicketNumber
				else
					local TicketString = tostring(TicketNumber)
					table.insert(glrCharacter[state].Entries.TicketNumbers[sender], TicketString)
					table.insert(glrTemp[state][sender].TicketNumbers, TicketString)
					if FirstTicketNumberBought then
						glrCharacter[state].Entries.TicketRange[sender].LowestTicketNumber = TicketNumber
						glrTemp[state][sender].TicketRange.LowestTicketNumber = TicketNumber
						FirstTicketNumberBought = false
					end
					if numTickets == i and LastTicketNumberBought then
						glrCharacter[state].Entries.TicketRange[sender].HighestTicketNumber = TicketNumber
						glrTemp[state][sender].TicketRange.HighestTicketNumber = TicketNumber
						LastTicketNumberBought = false
					end
					if guaranteedWinner then
						glrCharacter.Event[state].LastTicketBought = TicketNumber + 1
						TicketNumber = TicketNumber + 1
					else
						glrCharacter.Event[state].LastTicketBought = TicketNumber + 2
						TicketNumber = TicketNumber + 2
					end
				end
				
			end
			
		end
		
		GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderHigh:SetText(TicketNumber)
		GLR_Update:UpdateInfo()
		
		if glrCharacter[state].MessageStatus == nil then
			glrCharacter[state].MessageStatus = {}
		end
		if glrCharacter[state].MessageStatus[sender] == nil then
			glrCharacter[state].MessageStatus[sender] = { ["sentMessage"] = false, ["lastAlert"] = dateTime }
		end
		
		local state = "Lottery"
		local targetRealm
		local bypass = false
		local vN = sender
		
		if string.find(string.lower(sender), "-") ~= nil then
			targetRealm = string.sub(sender, string.find(sender, "-") + 1)
		else
			bypass = true
		end
		
		if PlayerRealmName == targetRealm and not bypass then
			vN = string.sub(sender, 1, string.find(sender, "-") - 1)
		end
		
		if continue then
			
			GLR:SendPlayerTheirTicketInfo(glrTemp, vN, state)
			
		elseif not continue and MultiGuild then
			
			--With Multi-Guild Events enabled, the mod will only attempt to send a message to the target player if they're part of the same faction.
			for key, val in pairs(glrHistory.GuildRoster) do
				for i = 1, #glrHistory.GuildRoster[key] do
					local SubText = glrHistory.GuildRoster[key][i]
					local CompareText = string.sub(SubText, 1, string.find(SubText, '%[') - 1)
					if CompareText == sender then
						local TargetFaction = glrHistory.GuildFaction[key]
						if TargetFaction == faction then
							continue = true
						end
						break
					end
				end
				if continue then
					break
				end
			end
			if continue then
				if not glrCharacter[state].MessageStatus[sender].sentMessage then
					local target = GLR_Libs.WhoLib:UserInfo(vN, 
						{
							timeout = 1,
							Online,
							callback = function(target)
								GLR:CheckMultiGuildPlayerStatus(target, glrTemp, vN, state)
							end
						}
					)
				end
			end
			
		elseif not continue and not MultiGuild then
			
			if glrCharacter.Data.Settings.ToggleChatInfo then
				GLR:Print("|cfff7ff4d" .. sender .. " " .. L["Send Info Player Not Online"] .. "|r")
			end
			
		end
		
	elseif glrCharacter[state].Entries.Tickets[sender] ~= nil then
		
		local currentTickets = glrCharacter[state].Entries.Tickets[sender].NumberOfTickets
		local ticketChange = numTickets + currentTickets
		local continueEdit = true
		
		if ticketChange > maxTickets then
			numTickets = 0
		elseif ticketChange == maxTickets then
			numTickets = maxTickets - currentTickets
			ticketChange = maxTickets
		end
		
		if numTickets == 0 then
			continueEdit = false
		end
		
		if continueEdit then
			
			local FirstTicketNumberBought = true
			local LastTicketNumberBought = true
			local TicketNumber = glrCharacter.Event[state].LastTicketBought
			
			for i = 1, numTickets do
				
				if glrCharacter[state].TicketPool[1] ~= nil then
					
					table.insert(glrCharacter[state].Entries.TicketNumbers[sender], glrCharacter[state].TicketPool[1])
					table.insert(glrTemp[state][sender], glrCharacter[state].TicketPool[1])
					if numTickets == i and LastTicketNumberBought then
						glrCharacter[state].Entries.TicketRange[sender].HighestTicketNumber = tonumber(glrCharacter[state].TicketPool[1])
						glrTemp[state][sender].TicketRange.HighestTicketNumber = tonumber(glrCharacter[state].TicketPool[1])
						LastTicketNumberBought = false
					end
					table.remove(glrCharacter[state].TicketPool, 1)
					
				else
					
					local ts = tostring(TicketNumber)
					table.insert(glrCharacter[state].Entries.TicketNumbers[sender], ts)
					table.insert(glrTemp[state][sender], ts)
					if numTickets == i and LastTicketNumberBought then
						glrCharacter[state].Entries.TicketRange[sender].HighestTicketNumber = TicketNumber
						glrTemp[state][sender].TicketRange.HighestTicketNumber = TicketNumber
						LastTicketNumberBought = false
					end
					if guaranteedWinner then
						glrCharacter.Event[state].LastTicketBought = TicketNumber + 1
						TicketNumber = TicketNumber + 1
					else
						glrCharacter.Event[state].LastTicketBought = TicketNumber + 2
						TicketNumber = TicketNumber + 2
					end
					
				end
				
			end
			
			sort(glrCharacter[state].Entries.TicketNumbers[sender])
			
			glrCharacter.Event[state].TicketsSold = glrCharacter.Event[state].TicketsSold + numTickets
			glrCharacter[state].Entries.Tickets[sender].NumberOfTickets = ticketChange
			glrTemp[state][sender].NumberOfTickets = ticketChange
			
			GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderHigh:SetText(TicketNumber)
			GLR_Update:UpdateInfo()
			
			local state = "Lottery"
			local targetRealm
			local bypass = false
			local vN = sender
			
			if string.find(string.lower(sender), "-") ~= nil then
				targetRealm = string.sub(sender, string.find(sender, "-") + 1)
			else
				bypass = true
			end
			
			if PlayerRealmName == targetRealm and not bypass then
				vN = string.sub(sender, 1, string.find(sender, "-") - 1)
			end
			
			if continue then
				
				GLR:SendPlayerTheirTicketInfo(glrTemp, vN, state)
				
			elseif not continue and MultiGuild then
				
				--With Multi-Guild Events enabled, the mod will only attempt to send a message to the target player if they're part of the same faction.
				for key, val in pairs(glrHistory.GuildRoster) do
					for i = 1, #glrHistory.GuildRoster[key] do
						local SubText = glrHistory.GuildRoster[key][i]
						local CompareText = string.sub(SubText, 1, string.find(SubText, '%[') - 1)
						if CompareText == sender then
							local TargetFaction = glrHistory.GuildFaction[key]
							if TargetFaction == faction then
								continue = true
							end
							break
						end
					end
					if continue then
						break
					end
				end
				if continue then
					if not glrCharacter[state].MessageStatus[sender].sentMessage then
						local target = GLR_Libs.WhoLib:UserInfo(vN, 
							{
								timeout = 1,
								Online,
								callback = function(target)
									GLR:CheckMultiGuildPlayerStatus(target, glrTemp, vN, state)
								end
							}
						)
					end
				end
				
			elseif not continue and not MultiGuild then
				
				if glrCharacter.Data.Settings.ToggleChatInfo then
					GLR:Print("|cfff7ff4d" .. sender .. " " .. L["Send Info Player Not Online"] .. "|r")
				end
				
			end
			
		end
		
	end
	
end

-- Frame to process opening mail
GLR_Mail.GLR_UpdateFrame = CreateFrame("Frame") -- Made a global frame so the rest of the mod can access it.
GLR_Mail.GLR_UpdateFrame:Hide()
GLR_Mail.GLR_UpdateFrame:SetScript("OnShow", function(self)
	self.time = 2
	if self.time < 1.0 and not self.lootingMoney then
		-- Delay opening to 1.5 second to account for a nearly full
		-- inventory to respect the KeepFreeSpace setting
		self.time = 2
	end
	self.lootingMoney = nil
end)
GLR_Mail.GLR_UpdateFrame:SetScript("OnUpdate", function(self, elapsed)
	self.time = self.time - elapsed
	if self.time <= 0 then
		self:Hide()
		GLR_Mail:ProcessNext()
	end
end)

function GLR_RefreshFrame:OnEvent(event)
	local current, total = GetInboxNumItems()
	if current == MAX_MAIL_SHOWN or current == total then
		self.time = 3
		self.mode = 1
		GLR:UnregisterEvent("MAIL_INBOX_UPDATE")
	end
end
GLR_RefreshFrame:SetScript("OnEvent", GLR_RefreshFrame.OnEvent)

function GLR_Mail:OpenAll(isRecursive)
	if (not MailFrame:IsVisible()) then
		return
	end
	
	GLR_RefreshFrame:Hide()
	mailIndex, numUnshownItems = GetInboxNumItems()
	numUnshownItems = numUnshownItems - mailIndex
	lastNumGold = nil
	wait = false
	
	if mailIndex == 0 then
		Reset()
		return
	end
	firstMailDaysLeft = select(7, GetInboxHeaderInfo(1))
	
	GLR_Mail:DisableInbox(1)
	
	GLR_UI.GLR_ScanMailButtonText:SetText(L["Mail Scan Processing"])
	
	GLR_Mail:ProcessNext()
end

function GLR_Mail:ProcessNext()
	
	local status = glrCharacter.Data.Settings.MultiGuild
	local FullPlayerName = (GetUnitName("PLAYER", false) .. "-" .. string.gsub(string.gsub(GetRealmName(), "-", ""), "%s+", ""))
	local PlayerRealmName = string.sub(FullPlayerName, string.find(FullPlayerName, "-") + 1)
	local LotteryStatus = glrCharacter.Data.Settings.CurrentlyActiveLottery
	local RaffleStatus = glrCharacter.Data.Settings.CurrentlyActiveRaffle
	local continue = false
	local currentFirstMailDaysLeft = select(7, GetInboxHeaderInfo(1))
	if currentFirstMailDaysLeft ~= firstMailDaysLeft then
		return GLR_Mail:OpenAll(true)
	end
	if LotteryStatus == nil then
		LotteryStatus = false
	end
	if RaffleStatus == nil then
		RaffleStatus = false
	end
	
	if glrRosterTotal == 0 then
		if not status then
			GLR_Update:UpdateRosterTable()
		else
			GLR_Update:UpdateMultiGuildTable()
		end
	end
	
	if mailIndex > 0 then
		
		if wait then
			local goldCount = CountMoney()
			if lastNumGold ~= goldCount then
				wait = false
				mailIndex = mailIndex - 1
				return self:ProcessNext() -- Tail Call
			else
				GLR_Mail.GLR_UpdateFrame:Show()
				return
			end
		end
		
		local _, _, sender, subject, money, CODAmount, _, hasItem, _, _, msgText, _, isGM = GetInboxHeaderInfo(mailIndex)
		
		if sender ~= nil then
			
			local name = sender
			
			if not string.find(string.lower(name), "-") ~= nil then
				sender = strjoin("-", name, PlayerRealmName)
			end
			
			if not status then
				for i = 1, #glrRoster do --Does this if Multi-Guild is Disabled
					local test = string.sub(glrRoster[i], 1, string.find(glrRoster[i], '%[') - 1)
					if sender == test then
						continue = true
						break -- No need to continue searching a potentially large table once a name's found
					end
				end
			else
				for i = 1, #glrMultiGuildRoster do --Does this if Multi-Guild is Enabled
					local test = string.sub(glrMultiGuildRoster[i], 1, string.find(glrMultiGuildRoster[i], '%[') - 1)
					if sender == test then
						continue = true
						break -- No need to continue searching a potentially large table once a name's found
					end
				end
			end
			
			--Skip mail if it contains a CoD or it's from a GM or if the sender isn't from your Guild (or Multi-Guild)
			if (CODAmount and CODAmount > 0) or (isGM) or not continue then
				mailIndex = mailIndex - 1
				return self:ProcessNext() -- Tail call
			end
			
			--Filter by mail type
			local mailType = GetMailType(subject)
			if mailType == "NonAHMail" then
				
				subject = string.lower(subject)
				
				if subject ~= string.lower(L["Raffle"]) and subject ~= string.lower(L["Lottery"]) then
					--Skips mails that aren't for Lottery or Raffle
					mailIndex = mailIndex - 1
					return self:ProcessNext() -- Tail call
				end
				
				if subject == string.lower(L["Raffle"]) and not RaffleStatus then
					--Skips Raffle mails if no Raffle is active
					mailIndex = mailIndex - 1
					return self:ProcessNext() -- Tail call
				end
				
				if subject == string.lower(L["Lottery"]) and not LotteryStatus then
					--Skips Lottery mails if no Lottery is active
					mailIndex = mailIndex - 1
					return self:ProcessNext() -- Tail call
				end
				
				if hasItem then
					--Skips mails with attachments
					mailIndex = mailIndex - 1
					return self:ProcessNext()
				end
				
			else
				--Skip AH Mail Types
				mailIndex = mailIndex - 1
				return self:ProcessNext() -- Tail call
			end
			
			if money > 0 then
				
				if InboxFrame:IsVisible() then
					
					local raffle = 0
					local numTickets = 0
					local ticketPrice = 0
					local printSubject = ""
					local moneyString = GetMoneyString(money)
					local ChatInfo = glrCharacter.Data.Settings.ToggleChatInfo
					
					if subject == string.lower(L["Raffle"]) or subject == "raffle" then --Detects the users local language of "Raffle" and also English "raffle"
						raffle = 1
						printSubject = L["Raffle"]
						ticketPrice = tonumber(glrCharacter.Event.Raffle.TicketPrice)
					elseif subject == string.lower(L["Lottery"]) or subject == "lottery" then --Detects the users local language of "Lottery" and also English "lottery"
						printSubject = L["Lottery"]
						raffle = 0
						ticketPrice = tonumber(glrCharacter.Event.Lottery.TicketPrice)
					else
						mailIndex = mailIndex - 1
						return self:ProcessNext() -- Tail Call
					end
					
					local totalTickets = moneyString / ticketPrice
					for i = 1, totalTickets do --Even if someone is short by 1 copper, it will not count that ticket. So gold sent must be a 1 to 1 ratio (gold to ticket cost)
						numTickets = numTickets + 1
					end
					
					if ChatInfo then
						GLR:Print(format("%s %s %s: %s. %s: %s", L["Mail Scan Processing"], sender, L["Mail Scan Format String 1"], printSubject, L["Mail Scan Format String 2"], moneyString))
					end
					
					TakeInboxMoney(mailIndex)
					
					if numTickets == 0 and ChatInfo then
						GLR:Print(sender .. " " .. L["Mail Scan Extra Ticket String 1"] .. " " .. moneyString .. " " .. L["Mail Scan Extra Ticket String 2"] .. ". " .. L["Mail Scan Extra Ticket String 3"] .. ".")
					end
					
					MailSetTickets(sender, numTickets, raffle)
					
					lastNumGold = CountMoney()
					
					wait = true
					
					GLR_Mail.GLR_UpdateFrame.lootingMoney = true
					GLR_Mail.GLR_UpdateFrame:Show()
					
				else
					
					GLR:Print(L["Mail Scan Interrupted"])
					
				end
				
			else
				--Mail has no money, even if it's labeled "Lottery" or "Raffle" it skips
				mailIndex = mailIndex - 1
				return self:ProcessNext() -- Tail call
			end
			
		else
			
			--Mail 'sender' name returned nil and would therefore cause a error, forces the scan to skip that mail. The user may click the Scan button again to attempt to process it again.
			mailIndex = mailIndex - 1
			return self:ProcessNext() -- Tail call
			
		end
		
	else
		
		local numItems, totalItems = GetInboxNumItems()
		if numUnshownItems ~= totalItems - numItems then
			return self:OpenAll(true) -- Tail call
		elseif totalItems > numItems and numItems < MAX_MAIL_SHOWN then
			GLR_RefreshFrame:Show()
			return
		end
		
		Reset()
		
	end
end

local function noop() end
function GLR_Mail:DisableInbox(disable)
	if disable then
		if not GLR:IsHooked("InboxFrame_OnClick") then
			GLR:RawHook("InboxFrame_OnClick", noop, true)
			for i = 1, 7 do
				_G["MailItem" .. i .. "ButtonIcon"]:SetDesaturated(1)
			end
		end
		GLR_UI.GLR_ScanMailButton:Disable()
	else
		if GLR:IsHooked("InboxFrame_OnClick") then
			GLR:Unhook("InboxFrame_OnClick")
			for i = 1, 7 do
				_G["MailItem" .. i .. "ButtonIcon"]:SetDesaturated(nil)
			end
		end
		GLR_UI.GLR_ScanMailButton:Enable()
	end
end