GLR_Options = {}

local L = LibStub("AceLocale-3.0"):GetLocale("GuildLotteryRaffle")
local dialog = LibStub("AceConfigDialog-3.0")
local PlayerName = GetUnitName("PLAYER", false)
local _, _, classIndex = UnitClass("PLAYER")
local FullPlayerName = (GetUnitName("PLAYER", false) .. "-" .. string.gsub(string.gsub(GetRealmName(), "-", ""), "%s+", ""))
local CopyLotteryProfileName = 0
local CopyRaffleProfileName = 0
local CopyLotteryMessageProfileName = 0
local CopyRaffleMessageProfileName = 0
local CopyBothMessageProfileName = 0
local GiveLotteryTickets = "0"
local GiveRaffleTickets = "0"
local ConfirmGiveLotteryTickets = false
local ConfirmGiveRaffleTickets = false
local CopyLotteryProfileTable = {}
local CopyRaffleProfileTable = {}
local CopyLotteryMessageProfileTable = {}
local CopyRaffleMessageProfileTable = {}
local CopyBothMessageProfileTable = {}
local classColors = {
    [1] = "C79C6E", -- Warrior
    [2] = "F58CBA", -- Paladin
    [3] = "ABD473", -- Hunter
    [4] = "FFF569", -- Rogue
    [5] = "FFFFFF", -- Priest
    [6] = "C41F3B", -- Death Knight
    [7] = "0070DE", -- Shaman
    [8] = "69CCF0", -- Mage
    [9] = "9482C9", -- Warlock
    [10]  = "00FF96", -- Monk
    [11]  = "FF7D0A", -- Druid
    [12]  = "A330C9" -- Demon Hunter
}

--[[
%lottery/raffle_price = Ticket Cost
%lottery/raffle_max = Max Tickets
%lottery/raffle_name = Event name
%lottery/raffle_date = Event Date
%lottery/raffle_gold = Lottery Jackpot or Raffle Tickets Sold
%lottery/raffle_reply = Detection phrase
]]

local function PreviewMessage(message, state)
	local chatColor = string.format("%02X", ChatTypeInfo["GUILD"].r * 255) .. string.format("%02X", ChatTypeInfo["GUILD"].g * 255) .. string.format("%02X", ChatTypeInfo["GUILD"].b * 255)
	local showTimestamps = GetCVar("showTimestamps")
	local DateTime = ""
	local UserName = "[Guild] [|r|cFF" .. classColors[classIndex] .. PlayerName .. "|r|cFF" .. chatColor .. "]: "
	if showTimestamps ~= "none" then
		DateTime = date(showTimestamps)
	end
	message = string.gsub(message, "%%lottery_reply", L["Detect Lottery"])
	message = string.gsub(message, "%%raffle_reply", L["Detect Raffle"])
	if state == "Lottery" or state == "Both" then
		if glrCharacter.Data.Settings.CurrentlyActiveLottery then
			local gold = tostring((tonumber(glrCharacter.Event.Lottery.TicketPrice) * glrCharacter.Event.Lottery.TicketsSold) - ((tonumber(glrCharacter.Event.Lottery.TicketPrice) * glrCharacter.Event.Lottery.TicketsSold) * (tonumber(glrCharacter.Event.Lottery.GuildCut) / 100)))
			local tickets = tostring(glrCharacter.Event.Lottery.TicketsSold)
			message = string.gsub(message, "%%lottery_gold", gold)
			message = string.gsub(message, "%%lottery_price", glrCharacter.Event.Lottery.TicketPrice)
			message = string.gsub(message, "%%lottery_max", glrCharacter.Event.Lottery.MaxTickets)
			message = string.gsub(message, "%%lottery_name", glrCharacter.Event.Lottery.EventName)
			message = string.gsub(message, "%%lottery_date", glrCharacter.Event.Lottery.Date)
			message = string.gsub(message, "%%lottery_tickets", tickets)
		end
	end
	if state == "Raffle" or state == "Both" then
		if glrCharacter.Data.Settings.CurrentlyActiveRaffle then
			local gold = tostring(glrCharacter.Event.Raffle.TicketsSold)
			message = string.gsub(message, "%%raffle_gold", gold)
			message = string.gsub(message, "%%raffle_price", glrCharacter.Event.Raffle.TicketPrice)
			message = string.gsub(message, "%%raffle_max", glrCharacter.Event.Raffle.MaxTickets)
			message = string.gsub(message, "%%raffle_name", glrCharacter.Event.Raffle.EventName)
			message = string.gsub(message, "%%raffle_date", glrCharacter.Event.Raffle.Date)
		end
	end
	return "|cFF" .. chatColor .. DateTime .. UserName .. message .. "|r"
end

function GLR_Options:OptionsTable()
	
	local options = {
		name = "Guild Lottery & Raffles",
		type = "group",
		handler = GLRoptions,
		get = get,
		set = set,
		args = {
			glrInfo = {
				order = 1,
				icon = "Interface\\ICONS\\INV_Misc_Coin_01",
				type = "group",
				name = L["Configuration Panel Information Tab"],
				childGroups = "tab",
				args = {
					version = {
						order = 1,
						type = "description",
						name = "|cFF87CEFAv" .. GLR_Version .. "|r",
					},
					versionAuthor = {
						order = 2,
						type = "description",
						name = "|cFFFFE300" .. L["Configuration Panel Mod Author"] .. " |r" .. "|cffffffffTempuseteri-Area52|r", 
					},
					versionPreviousAuthors = {
						order = 3,
						type = "description",
						name = "|cFFFFE300" .. L["Configuration Panel Previous Authors"] .. " |r" .. "|cffffffffReaper_029, Malaron|r",
					},
					general_Translators = {
						order = 4,
						type = "description",
						name = "|cFFFFE300" .. L["Configuration Panel Translators"] .. " |r" .. "|cffffffffSkoldgaster (Spanish), QCElysiums (French).|r",
					},
					general_toggleEnable = {
						order = 5,
						type = "toggle",
						name = L["Configuration Panel Toggle GLR"],
						desc = L["Configuration Panel Toggle GLR Description"],
						width = "double",
						get = function()
							return glrCharacter.Data.Settings.enabled
						end,
						set = function(info, value)
							glrCharacter.Data.Settings.enabled = value
						end,
					},
					slashCommandsHeader = {
						order = 6,
						type = "header",
						name = L["Configuration Panel Slash Command List"],
					},
					slashCommandGLRgui = {
						order = 7,
						type = "description",
						name = "|cFFFFE300/glr |r" .. "|cffffffffor|r" .. "|cFFFFE300 /glr open |r" .. "|cffffffff- " .. L["Configuration Panel Open GLR Command"] .. "|r",
					},
					slashCommandGLRhelp = {
						order = 8,
						type = "description",
						name = "|cFFFFE300/glr help |r" .. "|cffffffff- " .. L["Configuration Panel Help Command"] .. "|r",
					},
					slashCommandGLRconfig = {
						order = 9,
						type = "description",
						name = "|cFFFFE300/glr config |r" .. "|cffffffff- " .. L["Configuration Panel Config Command"] .. "|r",
					},
					general_spacer1 = {
						order = 10,
						type = "description",
						name = " ",
					},
					howToHeader = {
						order = 11,
						type = "header",
						name = L["Configuration Panel How To Header"],
					},
					howToStep1 = {
						order = 12,
						type = "description",
						name = "|cFF87CEFA" .. L["Configuration Panel How To Step 1 P1"] .. "|r: " .. L["Configuration Panel How To Step 1 P2"],
					},
					howToStep2 = {
						order = 13,
						type = "description",
						name = "|cFF87CEFA" .. L["Configuration Panel How To Step 2 P1"] .. "|r: " .. L["Configuration Panel How To Step 2 P2"],
					},
					howToStep3 = {
						order = 14,
						type = "description",
						name = "|cFF87CEFA" .. L["Configuration Panel How To Step 3 P1"] .. "|r: " .. L["Configuration Panel How To Step 3 P2"],
					},
					howToStep4 = {
						order = 15,
						type = "description",
						name = "|cFF87CEFA" .. L["Configuration Panel How To Step 4 P1"] .. "|r: " .. L["Configuration Panel How To Step 4 P2"],
					},
				},
			},
			generalOptions = {
				order = 2,
				type = "group",
				name = L["Configuration Panel Options Tab"],
				childGroups = "tab",
				get = get,
				set = set,
				args = {
					generalOptionsTab = {
						order = 1,
						type = "group",
						name = L["Configuration Panel Options Tab Config"],
						childGroups = "tab",
						args = {
							general_options_header = {
								order = 1,
								type = "header",
								name = L["Configuration Panel Options Header"],
							},
							general_glrOptionsTab = {
								order = 2,
								type = "group",
								name = L["Configuration Panel Options Tab General"],
								childGroups = "tab",
								get = get,
								set = set,
								args = {
									general_glrOptionsHeader = {
										order = 1,
										type = "header",
										name = L["Configuration Panel General Options Header"],
									},
									general_open_glr = {
										order = 2,
										type = "execute",
										name = L["Configuration Panel Options Open GLR Button"],
										desc = L["Configuration Panel Options Open GLR Button Desc"],
										width = "full",
										func = function()
											InterfaceOptionsFrame_Show()
											if GLR_UI.GLR_MainFrame:IsVisible() == false then
												GLR:ShowGLRFrame()
											end
										end,
									},
									general_toggleEscape = {
										order = 3,
										type = "toggle",
										name = L["Configuration Panel Options Toggle Escape Key"],
										desc = L["Configuration Panel Options Toggle Escape Key Desc"],
										width = "normal",
										get = function()
											return glrCharacter.Data.Settings.ToggleEscape
										end,
										set = function(info, value)
											glrCharacter.Data.Settings.ToggleEscape = value
											GLR_Update:ToggleEscapeKey()
										end,
									},
									general_toggleChatInfo = {
										order = 4,
										type = "toggle",
										name = L["Configuration Panel Options Toggle Chat Info"],
										desc = L["Configuration Panel Options Toggle Chat Info Desc"],
										width = "normal",
										get = function()
											return glrCharacter.Data.Settings.ToggleChatInfo
										end,
										set = function(info, value)
											glrCharacter.Data.Settings.ToggleChatInfo = value
										end,
									},
									general_toggleSendLargeInfo = {
										order = 5,
										type = "toggle",
										name = L["Configuration Panel Options Toggle Full Ticket Info"],
										desc = L["Configuration Panel Options Toggle Full Ticket Info Desc"],
										width = "normal",
										get = function()
											return glrCharacter.Data.Settings.SendLargeInfo
										end,
										set = function(info, value)
											glrCharacter.Data.Settings.SendLargeInfo = value
										end,
									},
									general_toggleMultiGuild = {
										order = 6,
										type = "toggle",
										name = L["Configuration Panel Options Toggle Multi Guild"],
										desc = L["Configuration Panel Options Toggle Multi Guild Desc"],
										width = "normal",
										get = function()
											return glrCharacter.Data.Settings.MultiGuild
										end,
										set = function(info, value)
											local status = false
											local ActiveEventOn = ""
											local OtherToonActive = false
											if glrCharacter.Data.Settings.CurrentlyActiveLottery then
												status = true
											elseif glrCharacter.Data.Settings.CurrentlyActiveRaffle then
												status = true
											end
											if not status then --Checks your other characters to see if they have events running, preventing you from un/linking your guild while a event is active.
												if glrHistory.ActiveEvents ~= nil then
													for name, event in pairs(glrHistory.ActiveEvents) do
														if event then
															status = true
															OtherToonActive = true
															ActiveEventOn = tostring(name)
															break
														end
													end
												end
											end
											if not status then --Prevents changing this value while a Lottery/Raffle is active
												glrCharacter.Data.Settings.MultiGuild = value
												if glrHistory.MultiGuildStatus[FullPlayerName] ~= nil then
													glrHistory.MultiGuildStatus[FullPlayerName] = value
												end
											else
												if not OtherToonActive then
													GLR:Print(L["Configuration Panel Options Toggle Multi Guild Failed"])
												else
													GLR:Print(ActiveEventOn .. " " .. L["Configuration Panel Options Toggle Multi Guild Failed Other Toon"] .. " " .. L["Configuration Panel Options Toggle Multi Guild Failed"])
												end
											end
											GLR_Update:GetLinkedNames()
											GLR_Update:GetLinkedGuilds()
											GLR_Update:UpdateMultiGuildTable()
										end,
									},
									general_toggleCrossFactionEvents = {
										order = 7,
										type = "toggle",
										name = L["Configuration Panel Options Toggle Cross Faction"],
										desc = L["Configuration Panel Options Toggle Cross Faction Desc"],
										width = "normal",
										get = function()
											return glrCharacter.Data.Settings.CrossFactionEvents
										end,
										set = function(info, value)
											local lottery = glrCharacter.Data.Settings.CurrentlyActiveLottery
											local raffle = glrCharacter.Data.Settings.CurrentlyActiveRaffle
											if not lottery and not raffle then --Prevents changing this value while a Lottery/Raffle is active
												glrCharacter.Data.Settings.CrossFactionEvents = value
											else
												GLR:Print(L["Configuration Panel Options Toggle Cross Faction Failed"])
											end
										end,
									},
									general_toggleCalendarEvents = {
										order = 8,
										type = "toggle",
										name = L["Configuration Panel Options Toggle Calendar Events"],
										desc = L["Configuration Panel Options Toggle Calendar Events Desc"],
										width = "double",
										get = function()
											return glrCharacter.Data.Settings.CreateCalendarEvents
										end,
										set = function(info, value)
											GLR_Calendar:Toggle()
										end,
									},
									general_rangeTimeBetweenAlerts = {
										order = 9,
										type = "range",
										name = L["Configuration Panel Options Time Between Alerts"],
										desc = L["Configuration Panel Options Time Between Alerts Desc"],
										width = "full",
										min = 30,
										max = 600,
										step = 1,
										get = function()
											return glrCharacter.Data.Settings.TimeBetweenAlerts
										end,
										set = function(info, value)
											glrCharacter.Data.Settings.TimeBetweenAlerts = value
										end,
									},
								},
							},
							general_MinimapTab = {
								order = 3,
								type = "group",
								name = L["Configuration Panel Options Tab Minimap"],
								childGroups = "tab",
								get = get,
								set = set,
								args = {
									general_MinimapHeader = {
									 order = 1,
									 type = "header",
									 name = L["Configuration Panel Options Minimap Header"],
									},
									general_toggleMinimap = {
										order = 2,
										type = "toggle",
										name = L["Configuration Panel Options Minimap Toggle"],
										desc = L["Configuration Panel Options Minimap Toggle Desc"],
										width = "double",
										get = function()
											return glrCharacter.Data.Settings.mini_map
										end,
										set = function(info, value)
											glrCharacter.Data.Settings.mini_map = value
											GLR_Update:UpdateMinimapIcon()
										end,
									},
									general_rangeMinimapX = {
										order = 3,
										type = "range",
										name = L["Configuration Panel Options Minimap X Range"],
										desc = L["Configuration Panel Options Minimap X Range Desc"],
										width = "full",
										min = -100,
										max = 100,
										step = 0.1,
										get = function()
											local n = tonumber(string.format("%.1f", glrCharacter.Data.Settings.MinimapXOffset))
											return n
										end,
										set = function(info, value)
											glrCharacter.Data.Settings.MinimapXOffset = value
											GLR_Update:UpdateMinimapIcon()
										end,
									},
									general_rangeMinimapY = {
										order = 4,
										type = "range",
										name = L["Configuration Panel Options Minimap Y Range"],
										desc = L["Configuration Panel Options Minimap Y Range Desc"],
										width = "full",
										min = -100,
										max = 100,
										step = 0.1,
										get = function ()
											local n = tonumber(string.format("%.1f", glrCharacter.Data.Settings.MinimapYOffset))
											return n
										end,
										set = function(info, value)
											glrCharacter.Data.Settings.MinimapYOffset = value
											GLR_Update:UpdateMinimapIcon()
										end,
									},
								},
							},
							general_MultiGuildsTab = {
								order = 6,
								type = "group",
								name = L["Configuration Panel Options Tab Multi-Guild"],
								childGroups = "tab",
								get = get,
								set = set,
								args = {
									general_MultiGuilds_Header = {
										order = 1,
										type = "header",
										name = L["Configuration Panel Options Guild Header"],
									},
									general_description_MultiGuilds = {
										order = 2,
										type = "description",
										name = L["Configuration Panel Options Multi-Guild Description"],
										width = "full",
									},
									general_multiselect_MultiGuilds_LinkedGuilds = {
										order = 3,
										type = "multiselect",
										name = L["Configuration Panel Options Multi-Guild Linked Guilds"],
										desc = L["Configuration Panel Options Multi-Guild Tooltip"],
										width = "full",
										values = glrTempGuilds,
										get = function(info, key)
											local guild = glrTempGuilds[key]
											local doBreak = false
											for i = 1, #glrHistory.Names do
												local name = glrHistory.Names[i]
												if glrHistory.CharacterGuilds[name] == guild then
													for t, v in pairs(glrHistory.CharacterGuilds) do
														if glrHistory.CharacterGuilds[name] == v then
															local status = glrHistory.MultiGuildStatus[name]
															glrHistory.GuildStatus[v] = status
															if status then
																doBreak = true
																return glrHistory.GuildStatus[v]
															end
														end
													end
												end
												if doBreak then
													break
												end
											end
										end,
										set = function(info, key, value)
											local guild = glrTempGuilds[key]
											local status = false
											local ActiveEventOn = ""
											local OtherToonActive = false
											if glrCharacter.Data.Settings.CurrentlyActiveLottery then
												status = true
											elseif glrCharacter.Data.Settings.CurrentlyActiveRaffle then
												status = true
											end
											if not status then --Checks your other characters to see if they have events running, preventing you from un/linking your guild while a event is active.
												if glrHistory.ActiveEvents ~= nil then
													for t, v in pairs(glrHistory.ActiveEvents) do
														if v then
															status = true
															OtherToonActive = true
															ActiveEventOn = tostring(t)
															break
														end
													end
												end
											end
											if not status then --Prevents changing this value while a Lottery/Raffle is active
												for i = 1, #glrHistory.Names do
													local name = glrHistory.Names[i]
													if glrHistory.CharacterGuilds[name] == guild then
														for t, v in pairs(glrHistory.CharacterGuilds) do
															if glrHistory.CharacterGuilds[name] == v then
																glrHistory.MultiGuildStatus[name] = value
																glrHistory.GuildStatus[v] = value
															end
														end
														if name == FullPlayerName then
															glrCharacter.Data.Settings.MultiGuild = value
														end
													end
												end
											else
												if not OtherToonActive then
													GLR:Print(L["Configuration Panel Options Toggle Multi Guild Failed"])
												else
													GLR:Print(ActiveEventOn .. " " .. L["Configuration Panel Options Toggle Multi Guild Failed Other Toon"] .. " " .. L["Configuration Panel Options Toggle Multi Guild Failed"])
												end
											end
											GLR_Update:GetLinkedNames()
											GLR_Update:GetLinkedGuilds()
										end,
									},
									general_multiselect_MultiGuilds_PlayerNames = {
										order = 4,
										type = "multiselect",
										name = L["Configuration Panel Options Multi-Guild Linked Characters"],
										desc = L["Configuration Panel Options Multi-Guild Tooltip"],
										width = "full",
										values = glrTempNames,
										get = function(info, key)
											local name = glrTempNames[key]
											return glrHistory.MultiGuildStatus[name]
										end,
										set = function(info, key, value)
											local name = glrTempNames[key]
											local status = false
											local ActiveEventOn = ""
											local OtherToonActive = false
											if glrCharacter.Data.Settings.CurrentlyActiveLottery then
												status = true
											elseif glrCharacter.Data.Settings.CurrentlyActiveRaffle then
												status = true
											end
											if not status then --Checks your other characters to see if they have events running, preventing you from un/linking your guild while a event is active.
												if glrHistory.ActiveEvents ~= nil then
													for t, v in pairs(glrHistory.ActiveEvents) do
														if v then
															status = true
															OtherToonActive = true
															ActiveEventOn = tostring(t)
															break
														end
													end
												end
											end
											if not status then --Prevents changing this value while a Lottery/Raffle is active
												if name ~= nil then
													glrHistory.MultiGuildStatus[name] = value
													if name == FullPlayerName then
														glrCharacter.Data.Settings.MultiGuild = value
													end
												end
											else
												if not OtherToonActive then
													GLR:Print(L["Configuration Panel Options Toggle Multi Guild Failed"])
												else
													GLR:Print(ActiveEventOn .. " " .. L["Configuration Panel Options Toggle Multi Guild Failed Other Toon"] .. " " .. L["Configuration Panel Options Toggle Multi Guild Failed"])
												end
											end
											GLR_Update:GetLinkedNames()
											GLR_Update:GetLinkedGuilds()
										end,
									},
								},
							},
						},
					},
					ticketInfoTab = {
						order = 2,
						type = "group",
						name = L["Configuration Panel Options Tab Ticket Info"],
						childGroups = "tab",
						args = {
							ticketInfo_header = {
								order = 1,
								type = "header",
								name = L["Configuration Panel Ticket Info Header"],
							},
							ticketInfo_SelectLotteryPlayer = {
								order = 2,
								type = "select",
								name = L["Configuration Panel Ticket Info Select Lottery Player"],
								desc = L["Configuration Panel Ticket Info Select Lottery Player Desc"],
								style = "dropdown",
								width = "normal",
								values = function()
									local t = {}
									local c = 0
									if glrCharacter ~= nil then
										if glrCharacter.Lottery ~= nil then
											if glrCharacter.Lottery.Entries ~= nil then
												if glrCharacter.Lottery.Entries.Names ~= nil then
													for id, val in pairs(glrCharacter.Lottery.Entries.Names) do
														table.insert(t, val)
														c = c + 1
													end
												end
											end
										end
									end
									if c == 0 then
										table.insert(t, L["No Entries Found"])
										return t
									else
										sort(t)
										return t
									end
								end,
								get = function()
									return GLR_SelectLotteryPlayer
								end,
								set = function(_, key)
									GLR_SelectLotteryPlayer = key
								end,
							},
							ticketInfo_ViewLotteryPlayer = {
								order = 3,
								type = "execute",
								name = L["Configuration Panel Ticket Info Lottery Button"],
								desc = L["Configuration Panel Ticket Info Lottery Button Desc"],
								width = "normal",
								func = function()
									GLR:ViewPlayerTickets("Lottery")
								end,
							},
							ticketInfo_Spacer1 = {
								order = 4,
								type = "description",
								name = "",
							},
							ticketInfo_SelectRafflePlayer = {
								order = 5,
								type = "select",
								name = L["Configuration Panel Ticket Info Select Raffle Player"],
								desc = L["Configuration Panel Ticket Info Select Raffle Player Desc"],
								style = "dropdown",
								width = "normal",
								values = function()
									local t = {}
									local c = 0
									if glrCharacter ~= nil then
										if glrCharacter.Raffle ~= nil then
											if glrCharacter.Raffle.Entries ~= nil then
												if glrCharacter.Raffle.Entries.Names ~= nil then
													for id, val in pairs(glrCharacter.Raffle.Entries.Names) do
														table.insert(t, val)
														c = c + 1
													end
												end
											end
										end
									end
									if c == 0 then
										table.insert(t, L["No Entries Found"])
										return t
									else
										sort(t)
										return t
									end
								end,
								get = function()
									return GLR_SelectRafflePlayer
								end,
								set = function(_, key)
									GLR_SelectRafflePlayer = key
								end,
							},
							ticketInfo_ViewRafflePlayer = {
								order = 6,
								type = "execute",
								name = L["Configuration Panel Ticket Info Raffle Button"],
								desc = L["Configuration Panel Ticket Info Raffle Button Desc"],
								width = "normal",
								func = function()
									GLR:ViewPlayerTickets("Raffle")
								end,
							},
						},
					},
				},
			},
			eventSettings = {
				order = 3,
				type = "group",
				name = L["Configuration Panel Events Tab"],
				childGroups = "tree",
				get = get,
				set = set,
				args = {
					lotterySettings = {
						order = 1,
						type = "group",
						name = L["Configuration Panel Options Lottery Header"],
						childGroups = "tab",
						get = get,
						set = set,
						args = {
							Lottery_Header = {
								order = 1,
								type = "header",
								name = L["Configuration Panel Options Lottery Header"],
							},
							Lottery_Spacer_0 = {
								order = 2,
								type = "description",
								name = "",
								width = "full",
							},
							Lottery_Button_Cancel = {
								order = 3,
								type = "execute",
								name = L["Configuration Panel Options Cancel Lottery"],
								desc = L["Configuration Panel Options Cancel Lottery Desc"],
								width = "normal",
								func = function()
									GLR:CancelCurrentEvent("Lottery")
									GLR_ConfirmCancelLottery = false
								end,
							},
							Lottery_Toggle_ConfirmCancel = {
								order = 4,
								type = "toggle",
								name = L["Confirm Cancel"],
								desc = L["Confirm Cancel Desc"],
								width = "normal",
								get = function()
									local status = glrCharacter.Data.Settings.CurrentlyActiveLottery
									if not status then
										return false
									end
									return GLR_ConfirmCancelLottery
								end,
								set = function(info, value)
									GLR_ConfirmCancelLottery = value
								end,
							},
							Lottery_Spacer_1 = {
								order = 5,
								type = "description",
								name = "",
								width = "full",
							},
							Lottery_Input_GiveTickets = {
								order = 6,
								type = "input",
								name = "Give Away Tickets",
								desc = "Enter a amount of tickets to give all Guild Members",
								width = "normal",
								get = function()
									return GiveLotteryTickets
								end,
								set = function(info, val)
									local text = tonumber(val)
									if text == nil then
										text = "0"
									else
										if text >= 0 then
											if string.find(val, '%.') then
												text = string.sub(val, 1, string.find(val, '%.') - 1)
											else
												text = tostring(text)
											end
										else
											text = "0"
										end
									end
									if glrCharacter.Data.Settings.CurrentlyActiveLottery then
										local MaxTickets = tonumber(glrCharacter.Event.Lottery.MaxTickets)
										if tonumber(val) > MaxTickets then
											text = tostring(MaxTickets)
										end
										GiveLotteryTickets = text
									else
										GiveLotteryTickets = text
									end
								end,
							},
							Lottery_Execute_GiveTickets = {
								order = 7,
								type = "execute",
								name = "Start Giveaway",
								desc = "Giving tickets away does not increase the Jackpot",
								width = "normal",
								func = function()
									if glrCharacter.Data.Settings.CurrentlyActiveLottery then
										if ConfirmGiveLotteryTickets then
											if tonumber(GiveLotteryTickets) >= 1 then
												ConfirmGiveLotteryTickets = false
												GLR:GiveEveryoneTickets(GiveLotteryTickets, "Lottery")
												GiveLotteryTickets = "0"
											end
										end
									else
										GiveLotteryTickets = "0"
										ConfirmGiveLotteryTickets = false
									end
								end,
							},
							Lottery_Toggle_GiveTickets = {
								order = 8,
								type = "toggle",
								name = L["Confirm Action"],
								desc = "Must be checked to give the entered amount of tickets to all Guild Members",
								width = "normal",
								get = function()
									return ConfirmGiveLotteryTickets
								end,
								set = function()
									if ConfirmGiveLotteryTickets then
										ConfirmGiveLotteryTickets = false
									else
										ConfirmGiveLotteryTickets = true
									end
								end,
							},
						},
					},
					raffleSettings = {
						order = 2,
						type = "group",
						name = L["Configuration Panel Options Raffle Header"],
						childGroups = "tab",
						get = get,
						set = set,
						args = {
							Raffle_Header = {
								order = 1,
								type = "header",
								name = L["Configuration Panel Options Raffle Header"],
							},
							Raffle_Button_Cancel = {
								order = 2,
								type = "execute",
								name = L["Configuration Panel Options Cancel Raffle"],
								desc = L["Configuration Panel Options Cancel Raffle Desc"],
								width = "normal",
								func = function()
									GLR:CancelCurrentEvent("Raffle")
									GLR_ConfirmCancelRaffle = false
								end,
							},
							Raffle_Toggle_ConfirmCancel = {
								order = 3,
								type = "toggle",
								name = L["Confirm Cancel"],
								desc = L["Confirm Cancel Desc"],
								width = "normal",
								get = function()
									local status = glrCharacter.Data.Settings.CurrentlyActiveRaffle
									if not status then
										return false
									end
									return GLR_ConfirmCancelRaffle
								end,
								set = function(info, value)
									GLR_ConfirmCancelRaffle = value
								end,
							},
							Raffle_Spacer_0 = {
								order = 4,
								type = "description",
								name = "",
								width = "full",
							},
							Raffle_Toggle_MultipleWinners = {
								order = 5,
								type = "toggle",
								name = L["Configuration Panel Options Allow Multiple Raffle Winners"],
								desc = L["Configuration Panel Options Allow Multiple Raffle Winners Desc"],
								width = "double",
								get = function()
									return glrCharacter.Data.Settings.AllowMultipleRaffleWinners
								end,
								set = function(info, value)
									glrCharacter.Data.Settings.AllowMultipleRaffleWinners = value
								end,
							},
							Raffle_Spacer_1 = {
								order = 6,
								type = "description",
								name = "",
								width = "full",
							},
							Raffle_Input_GiveTickets = {
								order = 7,
								type = "input",
								name = "Give Away Tickets",
								desc = "Enter a amount of tickets to give all Guild Members",
								width = "normal",
								get = function()
									return GiveRaffleTickets
								end,
								set = function(info, val)
									local text = tonumber(val)
									if text == nil then
										text = "0"
									else
										if text >= 0 then
											if string.find(val, '%.') then
												text = string.sub(val, 1, string.find(val, '%.') - 1)
											else
												text = tostring(text)
											end
										else
											text = "0"
										end
									end
									if glrCharacter.Data.Settings.CurrentlyActiveRaffle then
										local MaxTickets = tonumber(glrCharacter.Event.Raffle.MaxTickets)
										if tonumber(val) > MaxTickets then
											text = tostring(MaxTickets)
										end
										GiveRaffleTickets = text
									else
										GiveRaffleTickets = text
									end
								end,
							},
							Raffle_Execute_GiveTickets = {
								order = 8,
								type = "execute",
								name = "Start Giveaway",
								desc = "Giving tickets away does not increase gold received through ticket sales",
								width = "normal",
								func = function()
									if glrCharacter.Data.Settings.CurrentlyActiveRaffle then
										if ConfirmGiveRaffleTickets then
											if tonumber(GiveRaffleTickets) >= 1 then
												ConfirmGiveRaffleTickets = false
												GLR:GiveEveryoneTickets(GiveRaffleTickets, "Raffle")
												GiveRaffleTickets = "0"
											end
										end
									else
										GiveRaffleTickets = "0"
										ConfirmGiveRaffleTickets = false
									end
								end,
							},
							Raffle_Toggle_GiveTickets = {
								order = 9,
								type = "toggle",
								name = L["Confirm Action"],
								desc = "Must be checked to give the entered amount of tickets to all Guild Members",
								width = "normal",
								get = function()
									return ConfirmGiveRaffleTickets
								end,
								set = function()
									if ConfirmGiveRaffleTickets then
										ConfirmGiveRaffleTickets = false
									else
										ConfirmGiveRaffleTickets = true
									end
								end,
							},
						},
					},
				},
			},
			profileOptions = {
				order = 4,
				type = "group",
				name = L["Configuration Panel Profiles Tab"],
				childGroups = "tab",
				get = get,
				set = set,
				args = {
					general_ProfileHeader = {
						order = 1,
						type = "header",
						name = L["Configuration Panel Options Profile Header"],
					},
					general_Raffle_ProfileDescription = {
						order = 2,
						type = "description",
						name = L["Configuration Panel Options Profile Desc"],
						width = "full",
					},
					LotteryProfileTab = {
						order = 3,
						type = "group",
						name = L["Lottery"],
						childGroups = "tab",
						args = {
							SelectLotteryProfile = {
								order = 1,
								type = "select",
								name = L["Configuration Panel Options Profile Copy"],
								desc = L["Configuration Panel Options Profile Copy Lottery Desc"],
								style = "dropdown",
								width = "normal",
								values = function()
									local c = 0
									CopyLotteryProfileTable = {}
									if glrHistory ~= nil then
										if glrHistory.Profile ~= nil then
											if glrHistory.Profile.Lottery ~= nil then
												for id, val in pairs(glrHistory.Profile.Lottery) do
													if id ~= FullPlayerName then
														table.insert(CopyLotteryProfileTable, id)
														c = c + 1
													end
												end
											end
										end
									end
									if c >= 1 then
										sort(CopyLotteryProfileTable)
										return CopyLotteryProfileTable
									else
										table.insert(CopyLotteryProfileTable, L["No Entries Found"])
										return CopyLotteryProfileTable
									end
								end,
								get = function()
									return CopyLotteryProfileName
								end,
								set = function(_, key)
									CopyLotteryProfileName = key
									local name = CopyLotteryProfileTable[key]
									for t, v in pairs(glrHistory.Profile.Lottery[name]) do
										glrHistory.Profile.Lottery[FullPlayerName][t] = glrHistory.Profile.Lottery[name][t]
									end
								end,
							},
							Spacer_1 = {
								order = 2,
								type = "description",
								name = "",
								width = "full",
							},
							StartingGold = {
								order = 3,
								type = "input",
								name = L["Configuration Panel Options Lottery Profile Starting Gold"],
								desc = L["Configuration Panel Options Profile Enter Amount Desc"],
								width = "0.75",
								get = function()
									return glrCharacter.Data.Settings.Profile.Lottery.StartingGold
								end,
								set = function(info, value)
									local text = tonumber(value)
									if text == nil then
										text = "0"
									else
										if text >= 0 then
											text = tostring(text)
										else
											text = "0"
										end
									end
									glrCharacter.Data.Settings.Profile.Lottery.StartingGold = text
								end,
							},
							Spacer_2 = {
								order = 4,
								type = "description",
								name = "",
								width = "normal",
							},
							MaxTickets = {
								order = 5,
								type = "input",
								name = L["Configuration Panel Options Profile Max Tickets"],
								desc = L["Configuration Panel Options Profile Enter Amount Desc"],
								width = "0.75",
								get = function()
									return glrCharacter.Data.Settings.Profile.Lottery.MaxTickets
								end,
								set = function(info, value)
									local text = tonumber(value)
									if text == nil then
										text = "0"
									else
										if text >= 0 and text <= 50000 then
											text = tostring(text)
										else
											text = "0"
										end
									end
									glrCharacter.Data.Settings.Profile.Lottery.MaxTickets = text
								end,
							},
							Spacer_3 = {
								order = 6,
								type = "description",
								name = "",
								width = "full",
							},
							TicketPrice = {
								order = 7,
								type = "input",
								name = L["Configuration Panel Options Profile Ticket Price"],
								desc = L["Configuration Panel Options Profile Enter Amount Desc"],
								width = "0.75",
								get = function()
									return glrCharacter.Data.Settings.Profile.Lottery.TicketPrice
								end,
								set = function(info, value)
									local text = tonumber(value)
									if text == nil then
										text = "0"
									else
										if text >= 0 then
											text = tostring(text)
										else
											text = "0"
										end
									end
									glrCharacter.Data.Settings.Profile.Lottery.TicketPrice = text
								end,
							},
							Spacer_4 = {
								order = 8,
								type = "description",
								name = "",
								width = "normal",
							},
							SecondPlace = {
								order = 9,
								type = "input",
								name = L["Configuration Panel Options Lottery Profile Second Place"],
								desc = L["Configuration Panel Options Profile Enter Amount Desc"],
								width = "0.75",
								get = function()
									return glrCharacter.Data.Settings.Profile.Lottery.SecondPlace
								end,
								set = function(info, value)
									local text = tonumber(value)
									local compare = tonumber(glrCharacter.Data.Settings.Profile.Lottery.GuildCut)
									if text == nil then
										text = "0"
									else
										compare = text + compare
										if text >= 0 and text <= 100 and compare <= 100 then
											text = tostring(text)
										else
											text = "0"
										end
									end
									glrCharacter.Data.Settings.Profile.Lottery.SecondPlace = text
								end,
							},
							Spacer_5 = {
								order = 10,
								type = "description",
								name = "",
								width = "full",
							},
							GuildCut = {
								order = 11,
								type = "input",
								name = L["Configuration Panel Options Lottery Profile Guild Cut"],
								desc = L["Configuration Panel Options Profile Enter Amount Desc"],
								width = "0.75",
								get = function()
									return glrCharacter.Data.Settings.Profile.Lottery.GuildCut
								end,
								set = function(info, value)
									local text = tonumber(value)
									local compare = tonumber(glrCharacter.Data.Settings.Profile.Lottery.SecondPlace)
									if text == nil then
										text = "0"
									else
										compare = text + compare
										if text >= 0 and text <= 100 and compare <= 100 then
											text = tostring(text)
										else
											text = "0"
										end
									end
									glrCharacter.Data.Settings.Profile.Lottery.GuildCut = text
								end,
							},
							Spacer_6 = {
								order = 12,
								type = "description",
								name = "",
								width = "normal",
							},
							GuaranteeWinner = {
								order = 13,
								type = "toggle",
								name = L["Configuration Panel Options Lottery Profile Guarantee Winner"],
								desc = L["Configuration Panel Options Lottery Profile Toggle Desc"],
								width = "normal",
								get = function()
									return glrCharacter.Data.Settings.Profile.Lottery.Guaranteed
								end,
								set = function(info, value)
									glrCharacter.Data.Settings.Profile.Lottery.Guaranteed = value
								end
							},
						},
					},
					RaffleProfileTab = {
						order = 4,
						type = "group",
						name = L["Raffle"],
						childGroups = "tab",
						args = {
							SelectRaffleProfile = {
								order = 1,
								type = "select",
								name = L["Configuration Panel Options Profile Copy"],
								desc = L["Configuration Panel Options Profile Copy Raffle Desc"],
								style = "dropdown",
								width = "normal",
								values = function()
									local c = 0
									CopyRaffleProfileTable = {}
									if glrHistory ~= nil then
										if glrHistory.Profile ~= nil then
											if glrHistory.Profile.Raffle ~= nil then
												for id, val in pairs(glrHistory.Profile.Raffle) do
													if id ~= FullPlayerName then
														table.insert(CopyRaffleProfileTable, id)
														c = c + 1
													end
												end
											end
										end
									end
									if c >= 1 then
										sort(CopyRaffleProfileTable)
										return CopyRaffleProfileTable
									else
										table.insert(CopyRaffleProfileTable, L["No Entries Found"])
										return CopyRaffleProfileTable
									end
								end,
								get = function()
									return CopyRaffleProfileName
								end,
								set = function(_, key)
									CopyRaffleProfileName = key
									local name = CopyRaffleProfileTable[key]
									for t, v in pairs(glrHistory.Profile.Raffle[name]) do
										glrHistory.Profile.Raffle[FullPlayerName][t] = glrHistory.Profile.Raffle[name][t]
									end
								end,
							},
							Spacer_1 = {
								order = 2,
								type = "description",
								name = "",
								width = "full",
							},
							MaxTickets = {
								order = 3,
								type = "input",
								name = L["Configuration Panel Options Profile Max Tickets"],
								desc = L["Configuration Panel Options Profile Enter Amount Desc"],
								width = "0.75",
								get = function()
									return glrCharacter.Data.Settings.Profile.Raffle.MaxTickets
								end,
								set = function(info, value)
									local text = tonumber(value)
									if text == nil then
										text = "0"
									else
										text = tostring(text)
									end
									glrCharacter.Data.Settings.Profile.Raffle.MaxTickets = text
								end,
							},
							Spacer_2 = {
								order = 4,
								type = "description",
								name = "",
								width = "normal",
							},
							TicketPrice = {
								order = 5,
								type = "input",
								name = L["Configuration Panel Options Profile Ticket Price"],
								desc = L["Configuration Panel Options Profile Enter Amount Desc"],
								width = "0.75",
								get = function()
									return glrCharacter.Data.Settings.Profile.Raffle.TicketPrice
								end,
								set = function(info, value)
									local text = tonumber(value)
									if text == nil then
										text = "0"
									else
										text = tostring(text)
									end
									glrCharacter.Data.Settings.Profile.Raffle.TicketPrice = text
								end,
							},
							Spacer_3 = {
								order = 6,
								type = "description",
								name = "",
								width = "full",
							},
							InvalidItems = {
								order = 7,
								type = "toggle",
								name = L["Configuration Panel Options Raffle Profile Invalid Items"],
								desc = L["Configuration Panel Options Raffle Profile Toggle Desc"],
								width = "normal",
								get = function()
									return glrCharacter.Data.Settings.Profile.Raffle.InvalidItems
								end,
								set = function(value)
									glrCharacter.Data.Settings.Profile.Raffle.InvalidItems = value
								end,
							},
						},
					},
					MessageProfileTab = {
						order = 5,
						type = "group",
						name = L["Configuration Panel Message Tab"],
						childGroups = "tree",
						get = get,
						set = set,
						args = {
							LotteryMessageProfileTab = {
								order = 1,
								type = "group",
								name = L["Lottery"],
								childGroups = "tab",
								args = {
									SelectLotteryMessageProfile = {
										order = 1,
										type = "select",
										name = L["Configuration Panel Options Profile Copy"],
										desc = L["Configuration Panel Message Profile Copy Lottery Desc"],
										style = "dropdown",
										width = "normal",
										values = function()
											local c = 0
											CopyLotteryMessageProfileTable = {}
											if glrHistory ~= nil then
												if glrHistory.Profile ~= nil then
													if glrHistory.Profile.Messages ~= nil then
														if glrHistory.Profile.Messages.Lottery ~= nil then
															for id, val in pairs(glrHistory.Profile.Messages.Lottery) do
																if id ~= FullPlayerName then
																	table.insert(CopyLotteryMessageProfileTable, id)
																	c = c + 1
																end
															end
														end
													end
												end
											end
											if c >= 1 then
												sort(CopyLotteryMessageProfileTable)
												return CopyLotteryMessageProfileTable
											else
												table.insert(CopyLotteryMessageProfileTable, L["No Entries Found"])
												return CopyLotteryMessageProfileTable
											end
										end,
										get = function()
											return CopyLotteryMessageProfileName
										end,
										set = function(_, key)
											CopyLotteryMessageProfileName = key
											local name = CopyLotteryMessageProfileTable[key]
											for i = 1, #glrHistory.Profile.Messages.Lottery[name] - 1 do
												glrHistory.Profile.Messages.Lottery[FullPlayerName][i] = glrHistory.Profile.Messages.Lottery[name][i]
												glrCharacter.Data.Messages.GuildAlerts.Lottery[i] = glrHistory.Profile.Messages.Lottery[name][i]
											end
										end,
									},
								},
							},
							RaffleMessageProfileTab = {
								order = 2,
								type = "group",
								name = L["Raffle"],
								childGroups = "tab",
								args = {
									SelectRaffleMessageProfile = {
										order = 1,
										type = "select",
										name = L["Configuration Panel Options Profile Copy"],
										desc = L["Configuration Panel Message Profile Copy Raffle Desc"],
										style = "dropdown",
										width = "normal",
										values = function()
											local c = 0
											CopyRaffleMessageProfileTable = {}
											if glrHistory ~= nil then
												if glrHistory.Profile ~= nil then
													if glrHistory.Profile.Messages ~= nil then
														if glrHistory.Profile.Messages.Raffle ~= nil then
															for id, val in pairs(glrHistory.Profile.Messages.Raffle) do
																if id ~= FullPlayerName then
																	table.insert(CopyRaffleMessageProfileTable, id)
																	c = c + 1
																end
															end
														end
													end
												end
											end
											if c >= 1 then
												sort(CopyRaffleMessageProfileTable)
												return CopyRaffleMessageProfileTable
											else
												table.insert(CopyRaffleMessageProfileTable, L["No Entries Found"])
												return CopyRaffleMessageProfileTable
											end
										end,
										get = function()
											return CopyRaffleMessageProfileName
										end,
										set = function(_, key)
											CopyRaffleMessageProfileName = key
											local name = CopyRaffleMessageProfileTable[key]
											for i = 1, #glrHistory.Profile.Messages.Raffle[name] - 1 do
												glrHistory.Profile.Messages.Raffle[FullPlayerName][i] = glrHistory.Profile.Messages.Raffle[name][i]
												glrCharacter.Data.Messages.GuildAlerts.Raffle[i] = glrHistory.Profile.Messages.Raffle[name][i]
											end
										end,
									},
								},
							},
							BothMessageProfileTab = {
								order = 3,
								type = "group",
								name = L["Both"],
								childGroups = "tab",
								args = {
									SelectRaffleMessageProfile = {
										order = 1,
										type = "select",
										name = L["Configuration Panel Options Profile Copy"],
										desc = L["Configuration Panel Message Profile Copy Both Desc"],
										style = "dropdown",
										width = "normal",
										values = function()
											local c = 0
											CopyBothMessageProfileTable = {}
											if glrHistory ~= nil then
												if glrHistory.Profile ~= nil then
													if glrHistory.Profile.Messages ~= nil then
														if glrHistory.Profile.Messages.Both ~= nil then
															for id, val in pairs(glrHistory.Profile.Messages.Both) do
																if id ~= FullPlayerName then
																	table.insert(CopyBothMessageProfileTable, id)
																	c = c + 1
																end
															end
														end
													end
												end
											end
											if c >= 1 then
												sort(CopyBothMessageProfileTable)
												return CopyBothMessageProfileTable
											else
												table.insert(CopyBothMessageProfileTable, L["No Entries Found"])
												return CopyBothMessageProfileTable
											end
										end,
										get = function()
											return CopyBothMessageProfileName
										end,
										set = function(_, key)
											CopyBothMessageProfileName = key
											local name = CopyBothMessageProfileTable[key]
											for i = 1, #glrHistory.Profile.Messages.Both[name] - 1 do
												glrHistory.Profile.Messages.Both[FullPlayerName][i] = glrHistory.Profile.Messages.Both[name][i]
												glrCharacter.Data.Messages.GuildAlerts.Both[i] = glrHistory.Profile.Messages.Both[name][i]
											end
										end,
									},
								},
							},
						},
					},
				},
			},
			messageOptions = {
				order = 5,
				type = "group",
				name = L["Configuration Panel Message Tab"],
				childGroups = "tab",
				get = get,
				set = set,
				args = {
					GuildAlertMessageTab_Lottery = {
						order = 1,
						type = "group",
						name = L["Configuration Panel Message Tab Guild Alert (L) Config"],
						childGroups = "tree",
						args = {
							GuildAlert_Lottery_Header = {
								order = 1,
								type = "header",
								name = L["Configuration Panel Message Tab Guild Alert (L) Name"],
							},
							GuildAlert_Lottery_Enable = {
								order = 2,
								type = "toggle",
								name = L["Configuration Panel Message Enable Lottery"],
								width = "full",
								get = function()
									return glrCharacter.Data.Messages.GuildAlerts.Lottery[8]
								end,
								set = function(info, val)
									glrCharacter.Data.Messages.GuildAlerts.Lottery[8] = val
								end,
							},
							GuildAlert_Lottery_Settings = {
								order = 3,
								type = "group",
								name = L["Configuration Panel Message Settings"],
								desc = L["Configuration Panel Message Settings Desc"] .. "\n\n" .. L["Configuration Panel Message Tab Formats Lottery"],
								childGroups = "tab",
								args = {
									GuildAlert_Lottery_Formats = {
										order = 0,
										type = "description",
										name = L["Configuration Panel Message Tab Formats Lottery"],
									},
									GuildAlert_Lottery_1 = {
										order = 1,
										type = "input",
										name = L["Configuration Panel Message One"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Lottery[1]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Lottery[1] = val
											end
										end,
									},
									GuildAlert_Lottery_2 = {
										order = 2,
										type = "input",
										name = L["Configuration Panel Message Two"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Lottery[2]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Lottery[2] = val
											end
										end,
									},
									GuildAlert_Lottery_3 = {
										order = 3,
										type = "input",
										name = L["Configuration Panel Message Three"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Lottery[3]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Lottery[3] = val
											end
										end,
									},
									GuildAlert_Lottery_4 = {
										order = 4,
										type = "input",
										name = L["Configuration Panel Message Four"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Lottery[4]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Lottery[4] = val
											end
										end,
									},
									GuildAlert_Lottery_5 = {
										order = 5,
										type = "input",
										name = L["Configuration Panel Message Five"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Lottery[5]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Lottery[5] = val
											end
										end,
									},
									GuildAlert_Lottery_6 = {
										order = 6,
										type = "input",
										name = L["Configuration Panel Message Six"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Lottery[6]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Lottery[6] = val
											end
										end,
									},
									GuildAlert_Lottery_7 = {
										order = 7,
										type = "input",
										name = L["Configuration Panel Message Seven"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Lottery[7]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Lottery[7] = val
											end
										end,
									},
								},
							},
							GuildAlert_Lottery_Preview = {
								order = 4,
								type = "group",
								name = L["Configuration Panel Preview"],
								childGroups = "tab",
								args = {
									GuildAlert_Lottery_Preview_Header = {
										order = 0,
										type = "header",
										name = L["Configuration Panel Message Preview"],
									},
									GuildAlert_Lottery_Preview_1 = {
										order = 1,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Lottery[1], "Lottery") end,
									},
									GuildAlert_Lottery_Preview_2 = {
										order = 2,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Lottery[2], "Lottery") end,
									},
									GuildAlert_Lottery_Preview_3 = {
										order = 3,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Lottery[3], "Lottery") end,
									},
									GuildAlert_Lottery_Preview_4 = {
										order = 4,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Lottery[4], "Lottery") end,
									},
									GuildAlert_Lottery_Preview_5 = {
										order = 5,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Lottery[5], "Lottery") end,
									},
									GuildAlert_Lottery_Preview_6 = {
										order = 6,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Lottery[6], "Lottery") end,
									},
									GuildAlert_Lottery_Preview_7 = {
										order = 7,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Lottery[7], "Lottery") end,
									},
								},
							},
						},
					},
					GuildAlertMessageTab_Raffle = {
						order = 2,
						type = "group",
						name = L["Configuration Panel Message Tab Guild Alert (R) Config"],
						childGroups = "tree",
						args = {
							GuildAlert_Raffle_Header = {
								order = 1,
								type = "header",
								name = L["Configuration Panel Message Tab Guild Alert (R) Name"],
							},
							GuildAlert_Raffle_Enable = {
								order = 2,
								type = "toggle",
								name = L["Configuration Panel Message Enable Raffle"],
								width = "full",
								get = function()
									return glrCharacter.Data.Messages.GuildAlerts.Raffle[8]
								end,
								set = function(info, val)
									glrCharacter.Data.Messages.GuildAlerts.Raffle[8] = val
								end,
							},
							GuildAlert_Raffle_Settings = {
								order = 3,
								type = "group",
								name = L["Configuration Panel Message Settings"],
								desc = L["Configuration Panel Message Settings Desc"] .. "\n\n" .. L["Configuration Panel Message Tab Formats Raffle"],
								childGroups = "tab",
								args = {
									GuildAlert_Lottery_Formats = {
										order = 0,
										type = "description",
										name = L["Configuration Panel Message Tab Formats Raffle"],
									},
									GuildAlert_Raffle_1 = {
										order = 1,
										type = "input",
										name = L["Configuration Panel Message One"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Raffle[1]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Raffle[1] = val
											end
										end,
									},
									GuildAlert_Raffle_2 = {
										order = 2,
										type = "input",
										name = L["Configuration Panel Message Two"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Raffle[2]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Raffle[2] = val
											end
										end,
									},
									GuildAlert_Raffle_3 = {
										order = 3,
										type = "input",
										name = L["Configuration Panel Message Three"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Raffle[3]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Raffle[3] = val
											end
										end,
									},
									GuildAlert_Raffle_4 = {
										order = 4,
										type = "input",
										name = L["Configuration Panel Message Four"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Raffle[4]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Raffle[4] = val
											end
										end,
									},
									GuildAlert_Raffle_5 = {
										order = 5,
										type = "input",
										name = L["Configuration Panel Message Five"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Raffle[5]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Raffle[5] = val
											end
										end,
									},
									GuildAlert_Raffle_6 = {
										order = 6,
										type = "input",
										name = L["Configuration Panel Message Six"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Raffle[6]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Raffle[6] = val
											end
										end,
									},
									GuildAlert_Raffle_7 = {
										order = 7,
										type = "input",
										name = L["Configuration Panel Message Seven"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Raffle[7]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Raffle[7] = val
											end
										end,
									},
								},
							},
							GuildAlert_Raffle_Preview = {
								order = 4,
								type = "group",
								name = L["Configuration Panel Preview"],
								childGroups = "tab",
								args = {
									GuildAlert_Raffle_Preview_Header = {
										order = 0,
										type = "header",
										name = L["Configuration Panel Message Preview"],
									},
									GuildAlert_Raffle_Preview_1 = {
										order = 1,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Raffle[1], "Raffle") end,
									},
									GuildAlert_Raffle_Preview_2 = {
										order = 2,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Raffle[2], "Raffle") end,
									},
									GuildAlert_Raffle_Preview_3 = {
										order = 3,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Raffle[3], "Raffle") end,
									},
									GuildAlert_Raffle_Preview_4 = {
										order = 4,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Raffle[4], "Raffle") end,
									},
									GuildAlert_Raffle_Preview_5 = {
										order = 5,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Raffle[5], "Raffle") end,
									},
									GuildAlert_Raffle_Preview_6 = {
										order = 6,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Raffle[6], "Raffle") end,
									},
									GuildAlert_Raffle_Preview_7 = {
										order = 7,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Raffle[7], "Raffle") end,
									},
								},
							},
						},
					},
					GuildAlertMessageTab_Both = {
						order = 3,
						type = "group",
						name = L["Configuration Panel Message Tab Guild Alert (B) Config"],
						childGroups = "tree",
						args = {
							GuildAlert_Both_Header = {
								order = 1,
								type = "header",
								name = L["Configuration Panel Message Tab Guild Alert (B) Name"],
							},
							GuildAlert_Both_Enable = {
								order = 2,
								type = "toggle",
								name = L["Configuration Panel Message Enable Both"],
								desc = L["Configuration Panel Message Enable Both Desc"],
								width = "full",
								get = function()
									if not glrCharacter.Data.Messages.GuildAlerts.Lottery[8] then
										glrCharacter.Data.Messages.GuildAlerts.Both[12] = false
									end
									if not glrCharacter.Data.Messages.GuildAlerts.Raffle[8] then
										glrCharacter.Data.Messages.GuildAlerts.Both[12] = false
									end
									return glrCharacter.Data.Messages.GuildAlerts.Both[12]
								end,
								set = function(info, val)
									local lottery = glrCharacter.Data.Messages.GuildAlerts.Lottery[8]
									local raffle = glrCharacter.Data.Messages.GuildAlerts.Raffle[8]
									if lottery and raffle then
										glrCharacter.Data.Messages.GuildAlerts.Both[12] = val
									end
								end,
							},
							GuildAlert_Both_Settings = {
								order = 3,
								type = "group",
								name = L["Configuration Panel Message Settings"],
								desc = L["Configuration Panel Message Settings Desc"],
								childGroups = "tab",
								args = {
									GuildAlert_Both_1 = {
										order = 1,
										type = "input",
										name = L["Configuration Panel Message One"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[1]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[1] = val
											end
										end,
									},
									GuildAlert_Both_2 = {
										order = 2,
										type = "input",
										name = L["Configuration Panel Message Two"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[2]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[2] = val
											end
										end,
									},
									GuildAlert_Both_3 = {
										order = 3,
										type = "input",
										name = L["Configuration Panel Message Three"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[3]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[3] = val
											end
										end,
									},
									GuildAlert_Both_4 = {
										order = 4,
										type = "input",
										name = L["Configuration Panel Message Four"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[4]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[4] = val
											end
										end,
									},
									GuildAlert_Both_5 = {
										order = 5,
										type = "input",
										name = L["Configuration Panel Message Five"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[5]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[5] = val
											end
										end,
									},
									GuildAlert_Both_6 = {
										order = 6,
										type = "input",
										name = L["Configuration Panel Message Six"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[6]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[6] = val
											end
										end,
									},
									GuildAlert_Both_7 = {
										order = 7,
										type = "input",
										name = L["Configuration Panel Message Seven"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[7]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[7] = val
											end
										end,
									},
									GuildAlert_Both_8 = {
										order = 8,
										type = "input",
										name = L["Configuration Panel Message Eight"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[8]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[8] = val
											end
										end,
									},
									GuildAlert_Both_9 = {
										order = 9,
										type = "input",
										name = L["Configuration Panel Message Nine"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[9]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[9] = val
											end
										end,
									},
									GuildAlert_Both_10 = {
										order = 10,
										type = "input",
										name = L["Configuration Panel Message Ten"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[10]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[10] = val
											end
										end,
									},
									GuildAlert_Both_11 = {
										order = 11,
										type = "input",
										name = L["Configuration Panel Message Eleven"],
										width = "full",
										get = function()
											return glrCharacter.Data.Messages.GuildAlerts.Both[11]
										end,
										set = function(info, val)
											local text = string.len(val)
											if text > 255 then
												GLR:Print(L["Configuration Panel Message Too Long"])
											else
												glrCharacter.Data.Messages.GuildAlerts.Both[11] = val
											end
										end,
									},
								},
							},
							GuildAlert_Both_Preview = {
								order = 4,
								type = "group",
								name = L["Configuration Panel Preview"],
								childGroups = "tab",
								args = {
									GuildAlert_Both_Preview_Header = {
										order = 0,
										type = "header",
										name = L["Configuration Panel Message Preview"],
									},
									GuildAlert_Both_Preview_1 = {
										order = 1,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[1], "Both") end,
									},
									GuildAlert_Both_Preview_2 = {
										order = 2,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[2], "Both") end,
									},
									GuildAlert_Both_Preview_3 = {
										order = 3,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[3], "Both") end,
									},
									GuildAlert_Both_Preview_4 = {
										order = 4,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[4], "Both") end,
									},
									GuildAlert_Both_Preview_5 = {
										order = 5,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[5], "Both") end,
									},
									GuildAlert_Both_Preview_6 = {
										order = 6,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[6], "Both") end,
									},
									GuildAlert_Both_Preview_7 = {
										order = 7,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[7], "Both") end,
									},
									GuildAlert_Both_Preview_8 = {
										order = 8,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[8], "Both") end,
									},
									GuildAlert_Both_Preview_9 = {
										order = 9,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[9], "Both") end,
									},
									GuildAlert_Both_Preview_10 = {
										order = 10,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[10], "Both") end,
									},
									GuildAlert_Both_Preview_11 = {
										order = 11,
										type = "description",
										name = function(info) return PreviewMessage(glrCharacter.Data.Messages.GuildAlerts.Both[11], "Both") end,
									},
								},
							},
						},
					},
				},
			}
		},
	}
	return options
end