GLR_Update = {}

-- Purpose of this file is to centralize all functions into one lua file that relate to updating information that GLR uses.

--Global variables
glrRosterTotal = 0
glrMultiRosterTotal = 0

--Globally used tables
glrRoster = {} --Table used to hold list of guild members for adding players to a Lottery/Raffle event. Made global so other parts of the mod can access it (the GLR_Mail.lua file)
glrMultiGuildRoster = {} --Same as glrRoster but used to combine guild Rosters if Multi-Guild is enabled.
glrLinkedNames = {}
glrLinkedGuilds = {}

--Temporary Name and Guild table used to hold true/false status until reload/logout
glrTempNames = {}
glrTempGuilds = {}

local L = LibStub("AceLocale-3.0"):GetLocale("GuildLotteryRaffle")
local glrLinkedNamesCount = 0
local glrLinkedGuildsCount = 0
local FullPlayerName = (GetUnitName("PLAYER", false) .. "-" .. string.gsub(string.gsub(GetRealmName(), "-", ""), "%s+", ""))

--------------------------------------------- ---------
---------------- UPDATE INFO ----------------
---------------------------------------------
------- USED TO UPDATE DISPLAYED INFO -------
---------------------------------------------

function GLR_Update:UpdateInfo()
	
	local CheckGuild = GLR_Update:CheckGuildName()
	local guildName, _, _, _ = GetGuildInfo("PLAYER")
	local GetMonth = GLR:GetCurrentMonth()
	local GetDay = GLR:GetCurrentDay()
	local GetYear = GLR:GetCurrentYear()
	local GetHour = GLR:GetCurrentHour()
	local GetMinute = GLR:GetCurrentMinute()
	local status = glrCharacter.Data.Settings.Status
	local EditCalendarEvents = glrCharacter.Data.Settings.CreateCalendarEvents
	
	GLR_UI.GLR_NewLotteryEventFrame.GLR_ListOfMonthsSelectionText:SetText(GetMonth)
	GLR_UI.GLR_NewLotteryEventFrame.GLR_ListOfDaysSelectionText:SetText(GetDay)
	GLR_UI.GLR_NewLotteryEventFrame.GLR_ListOfYearsSelectionText:SetText(GetYear)
	GLR_UI.GLR_NewLotteryEventFrame.GLR_ListOfHoursSelectionText:SetText(GetHour)
	GLR_UI.GLR_NewLotteryEventFrame.GLR_ListOfMinutesSelectionText:SetText(GetMinute)
	
	GLR_UI.GLR_NewRaffleEventFrame.GLR_ListOfMonthsSelectionText:SetText(GetMonth)
	GLR_UI.GLR_NewRaffleEventFrame.GLR_ListOfDaysSelectionText:SetText(GetDay)
	GLR_UI.GLR_NewRaffleEventFrame.GLR_ListOfYearsSelectionText:SetText(GetYear)
	GLR_UI.GLR_NewRaffleEventFrame.GLR_ListOfHoursSelectionText:SetText(GetHour)
	GLR_UI.GLR_NewRaffleEventFrame.GLR_ListOfMinutesSelectionText:SetText(GetMinute)
	
	if CheckGuild then
		guildName, _, _, _ = GetGuildInfo("PLAYER")
	end
	
	if guildName == nil then
		guildName = "GLR - No Guild"
	end
	
	if glrCharacter.Data.Settings.GuildName == nil then
		glrCharacter.Data.Settings.GuildName = guildName
	end
	
	if status == "Raffle" then -- Displays Raffle data
		
		if not glrCharacter.Data.Settings.CurrentlyActiveRaffle or glrCharacter.Data.Settings.CurrentlyActiveRaffle == nil then
			
			GLR_UI.GLR_RaffleInfo.GLR_RaffleDateEditBox:SetText(glrCharacter.Data.Defaults.LotteryDate)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleDateEditBox:SetCursorPosition(0)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleNameEditBox:SetText(glrCharacter.Data.Defaults.RaffleName)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleNameEditBox:SetCursorPosition(0)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleMaxTicketsEditBox:SetText(glrCharacter.Data.Defaults.RaffleMaxTickets)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleTicketPriceEditBox:SetText(glrCharacter.Data.Defaults.TicketPrice)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleFirstPlacePrizeEditBox:SetText("")
			GLR_UI.GLR_RaffleInfo.GLR_RaffleSecondPlacePrizeEditBox:SetText("")
			GLR_UI.GLR_RaffleInfo.GLR_RaffleThirdPlacePrizeEditBox:SetText("")
			GLR_UI.GLR_RaffleInfo.GLR_RaffleTicketSalesEditBox:SetText("")
			GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderLow:SetText("0")
			GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderHigh:SetText("0")
			
			--Disable Certain Buttons
			GLR_UI.GLR_MainFrame.GLR_RaffleFrame.GLR_BeginRaffleButton:Disable()
			GLR_UI.GLR_MainFrame.GLR_RaffleFrame.GLR_AddPlayerToRaffleButton:Disable()
			GLR_UI.GLR_MainFrame.GLR_RaffleFrame.GLR_EditPlayerInRaffleButton:Disable()
			
		else
			
			local tickets = glrCharacter.Event.Raffle.TicketsSold
			local cost = tonumber(glrCharacter.Event.Raffle.TicketPrice)
			local ticketsSold = glrCharacter.Event.Raffle.LastTicketBought - 1
			local gold = tostring(cost * tickets)
			
			if tickets == 0 then
				ticketsSold = glrCharacter.Event.Raffle.FirstTicketNumber
			end
			
			GLR_UI.GLR_RaffleInfo.GLR_RaffleDateEditBox:SetText(glrCharacter.Event.Raffle.Date)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleDateEditBox:SetCursorPosition(0)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleNameEditBox:SetText(glrCharacter.Event.Raffle.EventName)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleNameEditBox:SetCursorPosition(0)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleMaxTicketsEditBox:SetText(glrCharacter.Event.Raffle.MaxTickets)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleTicketPriceEditBox:SetText(glrCharacter.Event.Raffle.TicketPrice)
			if glrCharacter.Event.Raffle.FirstPlace then
				GLR_UI.GLR_RaffleInfo.GLR_RaffleFirstPlacePrizeEditBox:SetText(glrCharacter.Event.Raffle.FirstPlace)
				GLR_UI.GLR_RaffleInfo.GLR_RaffleFirstPlacePrizeEditBox:SetCursorPosition(0)
			end
			if glrCharacter.Event.Raffle.SecondPlace then
				GLR_UI.GLR_RaffleInfo.GLR_RaffleSecondPlacePrizeEditBox:SetText(glrCharacter.Event.Raffle.SecondPlace)
				GLR_UI.GLR_RaffleInfo.GLR_RaffleSecondPlacePrizeEditBox:SetCursorPosition(0)
			else
				GLR_UI.GLR_RaffleInfo.GLR_RaffleSecondPlacePrizeEditBox:SetText("")
			end
			if glrCharacter.Event.Raffle.ThirdPlace then
				GLR_UI.GLR_RaffleInfo.GLR_RaffleThirdPlacePrizeEditBox:SetText(glrCharacter.Event.Raffle.ThirdPlace)
				GLR_UI.GLR_RaffleInfo.GLR_RaffleThirdPlacePrizeEditBox:SetCursorPosition(0)
			else
				GLR_UI.GLR_RaffleInfo.GLR_RaffleThirdPlacePrizeEditBox:SetText("")
			end
			GLR_UI.GLR_RaffleInfo.GLR_RaffleTicketSalesEditBox:SetText(gold)
			GLR_UI.GLR_RaffleInfo.GLR_RaffleTicketSalesEditBox:SetCursorPosition(0)
			GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderLow:SetText(glrCharacter.Event.Raffle.FirstTicketNumber)
			GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderHigh:SetText(ticketsSold)
			
			--Reenable Certain Buttons
			GLR_UI.GLR_MainFrame.GLR_RaffleFrame.GLR_BeginRaffleButton:Enable()
			GLR_UI.GLR_MainFrame.GLR_RaffleFrame.GLR_AddPlayerToRaffleButton:Enable()
			GLR_UI.GLR_MainFrame.GLR_RaffleFrame.GLR_EditPlayerInRaffleButton:Enable()
			
		end
		
	elseif status == "Lottery" then -- Displays Lottery data
		
		if not glrCharacter.Data.Settings.CurrentlyActiveLottery or glrCharacter.Data.Settings.CurrentlyActiveLottery == nil then
			
			GLR_UI.GLR_LotteryInfo.GLR_LotteryDateEditBox:SetText(glrCharacter.Data.Defaults.LotteryDate)
			GLR_UI.GLR_LotteryInfo.GLR_LotteryDateEditBox:SetCursorPosition(0)
			GLR_UI.GLR_LotteryInfo.GLR_LotteryNameEditBox:SetText(glrCharacter.Data.Defaults.LotteryName)
			GLR_UI.GLR_LotteryInfo.GLR_LotteryNameEditBox:SetCursorPosition(0)
			if glrCharacter.Data.Settings.PreviousLottery.Jackpot > 0 then
				local previousStartingGold = tostring(glrCharacter.Data.Settings.PreviousLottery.Jackpot)
				GLR_UI.GLR_LotteryInfo.GLR_StartingGoldEditBox:SetText(previousStartingGold)
				GLR_UI.GLR_LotteryInfo.GLR_JackpotEditBox:SetText(previousStartingGold)
			else
				GLR_UI.GLR_LotteryInfo.GLR_StartingGoldEditBox:SetText(glrCharacter.Data.Defaults.StartingGold)
				GLR_UI.GLR_LotteryInfo.GLR_JackpotEditBox:SetText(glrCharacter.Data.Defaults.LotteryJackpot)
			end
			GLR_UI.GLR_LotteryInfo.GLR_MaxTicketsEditBox:SetText(glrCharacter.Data.Defaults.LotteryMaxTickets)
			GLR_UI.GLR_LotteryInfo.GLR_TicketPriceEditBox:SetText(glrCharacter.Data.Defaults.TicketPrice)
			GLR_UI.GLR_LotteryInfo.GLR_SecondPlaceEditBox:SetText(glrCharacter.Data.Defaults.SecondCut)
			GLR_UI.GLR_LotteryInfo.GLR_GuildCutEditBox:SetText(glrCharacter.Data.Defaults.GuildCut)
			GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderLow:SetText("0")
			GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderHigh:SetText("0")
			
			--Disable Certain Buttons
			GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_BeginLotteryButton:Disable()
			GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_AddPlayerToLotteryButton:Disable()
			GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_EditPlayerInLotteryButton:Disable()
			GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_DonationButton:Disable()
			
		else
			
			local startingGold = tonumber(glrCharacter.Event["Lottery"]["StartingGold"])
			local tickets = glrCharacter.Event["Lottery"]["TicketsSold"]
			local price = tonumber(glrCharacter.Event["Lottery"]["TicketPrice"])
			local jackpotGold = tostring(startingGold + ( tickets * price ))
			local ticketsSold = glrCharacter.Event.Lottery.LastTicketBought - 1
			local guaranteedWinner = false
			
			if glrCharacter.Event.Lottery.SecondPlace == L["Winner Guaranteed"] then
				guaranteedWinner = true
			end
			
			if tickets == 0 then
				ticketsSold = glrCharacter.Event.Lottery.FirstTicketNumber
			end
			
			if not guaranteedWinner and tickets > 0 then
				ticketsSold = glrCharacter.Event.Lottery.LastTicketBought - 2
			end
			
			GLR_UI.GLR_LotteryInfo.GLR_LotteryDateEditBox:SetText(glrCharacter.Event.Lottery.Date)
			GLR_UI.GLR_LotteryInfo.GLR_LotteryDateEditBox:SetCursorPosition(0)
			GLR_UI.GLR_LotteryInfo.GLR_LotteryNameEditBox:SetText(glrCharacter.Event.Lottery.EventName)
			GLR_UI.GLR_LotteryInfo.GLR_LotteryNameEditBox:SetCursorPosition(0)
			GLR_UI.GLR_LotteryInfo.GLR_StartingGoldEditBox:SetText(glrCharacter.Event.Lottery.StartingGold)
			GLR_UI.GLR_LotteryInfo.GLR_JackpotEditBox:SetText(jackpotGold)
			GLR_UI.GLR_LotteryInfo.GLR_MaxTicketsEditBox:SetText(glrCharacter.Event.Lottery.MaxTickets)
			GLR_UI.GLR_LotteryInfo.GLR_TicketPriceEditBox:SetText(glrCharacter.Event.Lottery.TicketPrice)
			GLR_UI.GLR_LotteryInfo.GLR_SecondPlaceEditBox:SetText(glrCharacter.Event.Lottery.SecondPlace)
			GLR_UI.GLR_LotteryInfo.GLR_GuildCutEditBox:SetText(glrCharacter.Event.Lottery.GuildCut)
			GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderLow:SetText(glrCharacter.Event.Lottery.FirstTicketNumber)
			GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderHigh:SetText(ticketsSold)
			
			--Reenable Certain Buttons
			GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_BeginLotteryButton:Enable()
			GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_AddPlayerToLotteryButton:Enable()
			GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_EditPlayerInLotteryButton:Enable()
			GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_DonationButton:Enable()
			
		end
		
	end
	
	GLR_Update:UpdateRosterTable()
	GLR_Update:UpdateMultiGuildTable()
	GLR_Update:PopulateMultiGuildTable()
	GLR:UpdatePlayersAndTotalTickets()
	GLR:UpdateNumberOfUnusedTickets()
	
end

function GLR_Update:CheckGuildName()
	
	local GuildStatus = true
	
	if IsInGuild() then
		local test = GetGuildInfo("PLAYER")
		if test == nil then
			GuildStatus = false
		end
	end
	
	if GuildStatus then
		GuildRoster()
	end
	
	return GuildStatus
	
end

---------------------------------------------
------------ UPDATE MINIMAP ICON ------------
---------------------------------------------
------------ TOGGLES MINIMAP ICON -----------
---------------------------------------------

function GLR_Update:UpdateMinimapIcon()
	
	local status = glrCharacter.Data.Settings.mini_map
	local X = 0
	local Y = 0
	
	if glrCharacter.Data.Settings.MinimapXOffset ~= nil then
		X = glrCharacter.Data.Settings.MinimapXOffset
	end
	if glrCharacter.Data.Settings.MinimapYOffset ~= nil then
		Y = glrCharacter.Data.Settings.MinimapYOffset
	end
	
	if status then
		GLR_UI.GLR_MinimapButton:Show()
	else
		GLR_UI.GLR_MinimapButton:Hide()
	end
	
	GLR_UI.GLR_MinimapButton:SetPoint("CENTER", Minimap, "CENTER", X, Y)
	
end

--Sets whether or not frames used by GLR can be closed with the Escape key.
--Frames that don't have a close button will always be able to close with the Escape key, this is done in the GLR_UI.lua file in the Scripts section.
function GLR_Update:ToggleEscapeKey()
	
	local status = glrCharacter.Data.Settings.ToggleEscape
	
	if status then
		
		GLR_UI.GLR_MainFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_MainFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_MainFrame:SetPropagateKeyboardInput(false)
				GLR_UI.GLR_MainFrame:Hide()
			end
		end)
		
		GLR_UI.GLR_NewLotteryEventFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_NewLotteryEventFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_NewLotteryEventFrame:SetPropagateKeyboardInput(false)
				GLR_UI.GLR_NewLotteryEventFrame:Hide()
			end
		end)
		
		GLR_UI.GLR_AddPlayerToLotteryFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_AddPlayerToLotteryFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_AddPlayerToLotteryFrame:SetPropagateKeyboardInput(false)
				GLR_UI.GLR_AddPlayerToLotteryFrame:Hide()
			end
			
		end)
		
		GLR_UI.GLR_EditPlayerInLotteryFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_EditPlayerInLotteryFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_EditPlayerInLotteryFrame:SetPropagateKeyboardInput(false)
				GLR_UI.GLR_EditPlayerInLotteryFrame:Hide()
			end
		end)
		
		GLR_UI.GLR_NewRaffleEventFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_NewRaffleEventFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_NewRaffleEventFrame:SetPropagateKeyboardInput(false)
				GLR_UI.GLR_NewRaffleEventFrame:Hide()
			end
		end)
		
		GLR_UI.GLR_AddPlayerToRaffleFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_AddPlayerToRaffleFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_AddPlayerToRaffleFrame:SetPropagateKeyboardInput(false)
				GLR_UI.GLR_AddPlayerToRaffleFrame:Hide()
			end
		end)
		
		GLR_UI.GLR_EditPlayerInRaffleFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_EditPlayerInRaffleFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_EditPlayerInRaffleFrame:SetPropagateKeyboardInput(false)
				GLR_UI.GLR_EditPlayerInRaffleFrame:Hide()
			end
		end)
		
		GLR_UI.GLR_DonationFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_DonationFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_DonationFrame:SetPropagateKeyboardInput(false)
				GLR_UI.GLR_DonationFrame:Hide()
			end
		end)
		
	elseif not status then
		
		GLR_UI.GLR_MainFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_MainFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_MainFrame:SetPropagateKeyboardInput(true)
			end
		end)
		
		GLR_UI.GLR_NewLotteryEventFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_NewLotteryEventFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_NewLotteryEventFrame:SetPropagateKeyboardInput(true)
			end
		end)
		
		GLR_UI.GLR_AddPlayerToLotteryFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_AddPlayerToLotteryFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_AddPlayerToLotteryFrame:SetPropagateKeyboardInput(true)
			end
		end)
		
		GLR_UI.GLR_EditPlayerInLotteryFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_EditPlayerInLotteryFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_EditPlayerInLotteryFrame:SetPropagateKeyboardInput(true)
			end
		end)
		
		GLR_UI.GLR_NewRaffleEventFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_NewRaffleEventFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_NewRaffleEventFrame:SetPropagateKeyboardInput(true)
			end
		end)
		
		GLR_UI.GLR_AddPlayerToRaffleFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_AddPlayerToRaffleFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_AddPlayerToRaffleFrame:SetPropagateKeyboardInput(true)
			end
		end)
		
		GLR_UI.GLR_EditPlayerInRaffleFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_EditPlayerInRaffleFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_EditPlayerInRaffleFrame:SetPropagateKeyboardInput(true)
			end
		end)
		
		GLR_UI.GLR_DonationFrame:SetScript("OnKeyDown", function(_, key)
			GLR_UI.GLR_DonationFrame:SetPropagateKeyboardInput(true)
			if key == "ESCAPE" then
				GLR_UI.GLR_DonationFrame:SetPropagateKeyboardInput(true)
			end
		end)
		
	end
	
end

function GLR_Update:UpdateRosterTable()
	
	if glrRosterTotal > 0 then
		glrRoster = {} -- Resets the Roster Table so as to prevent multiple names being inserted.
		glrRosterTotal = 0
	end
	
	GuildRoster()
	
	local numTotalMembers, _, _ = GetNumGuildMembers()
	
	for i = 1, numTotalMembers do
		
		local name, _, _, level, class, _, _, _, _, _, _, _, _, _, _, _ = GetGuildRosterInfo(i)
		local text = name .. "[" .. class .. "]" .. "(" .. level .. ")"
		
		table.insert(glrRoster, text)
		
		glrRosterTotal = glrRosterTotal + 1
		
	end
	
	sort(glrRoster)
	
end

function GLR_Update:UpdateMultiGuildTable()
	
	GuildRoster()
	--local t = {
	--	"Emiliakaija-Sargeras[Rogue](120)" -- [1]
	--}
	local t = {}
	local guild = glrCharacter.Data.Settings.GuildName
	local numTotalMembers, _, _ = GetNumGuildMembers()
	local count = 0
	
	for i = 1, numTotalMembers do
		
		local name, _, _, level, class, _, _, _, _, _, _, _, _, _, _, _ = GetGuildRosterInfo(i)
		local text = name .. "[" .. class .. "]" .. "(" .. level .. ")"
		
		table.insert(t, text)
		if count == 0 then
			count = 1
		end
		
	end
	
	sort(t)
	
	if count > 0 then --Prevents clearing an already populated table when first logging in.
		
		if guild ~= nil then
			if glrHistory.GuildRoster[guild] == nil then
				glrHistory.GuildRoster[guild] = {}
				glrHistory.GuildRoster[guild] = t
			else
				--Clears the table for new names (or those that have left your guild)
				glrHistory.GuildRoster[guild] = t
			end
		end
		
	end
	
end

function GLR_Update:PopulateMultiGuildTable() --Used to populate the table the mod will now use (if Multi-Guild is enabled) for allowing player entry into a Lottery/Raffle.
	
	local processedGuilds = {}
	local CrossFaction = glrCharacter.Data.Settings.CrossFactionEvents
	local faction = glrHistory.PlayerFaction[FullPlayerName]
	local continue = true
	
	if faction == nil then return end
	
	if glrMultiRosterTotal > 0 then
		glrMultiGuildRoster = {} --Clears the multiguild table in case people have joined/left the currently logged in players Guild.
		glrMultiRosterTotal = 0
	end
	
	for i = 1, #glrHistory.Names do
		local continue = false
		local name = glrHistory.Names[i]
		local status = glrHistory.MultiGuildStatus[name]
		local guild = glrHistory.CharacterGuilds[name]
		local guildstatus = glrHistory.GuildStatus[guild]
		local guildfaction = glrHistory.GuildFaction[guild]
		
		glrMultiRosterTotal = glrMultiRosterTotal + 1
		
		if glrHistory.GuildRoster[guild] ~= nil then
			if status then
				if guildstatus then
					if processedGuilds[guild] == nil then
						if CrossFaction then
							processedGuilds[guild] = true
							for j = 1, #glrHistory.GuildRoster[guild] do
								table.insert(glrMultiGuildRoster, glrHistory.GuildRoster[guild][j])
							end
						else
							if guildfaction == faction then
								processedGuilds[guild] = true
								for j = 1, #glrHistory.GuildRoster[guild] do
									table.insert(glrMultiGuildRoster, glrHistory.GuildRoster[guild][j])
								end 
							end
						end
					end
				end
			end
		end
	end
	
	sort(glrMultiGuildRoster)
	
end

function GLR_Update:GetLinkedNames()
	
	if glrLinkedNamesCount > 0 then
		glrLinkedNames = {}
		glrLinkedNamesCount = 0
	end
	
	for i = 1, #glrHistory.Names do
		
		local name = glrHistory.Names[i]
		
		if glrHistory.MultiGuildStatus[name] then
			glrLinkedNames[name] = name
			glrLinkedNamesCount = glrLinkedNamesCount + 1
			if glrTempNames[name] == nil then
				glrTempNames[name] = name
			end
		end
		
	end
	
	sort(glrLinkedNames)
	sort(glrTempNames)
	
end

function GLR_Update:GetLinkedGuilds()
	
	if glrLinkedGuildsCount > 0 then
		glrLinkedGuilds = {}
		glrLinkedGuildsCount = 0
	end
	
	for i = 1, #glrHistory.Names do
		
		local name = glrHistory.Names[i]
		local guild = glrHistory.CharacterGuilds[name]
		
		if guild ~= nil then
			if glrHistory.MultiGuildStatus[name] then
				if glrLinkedGuilds[guild] == nil then
					glrLinkedGuilds[guild] = guild
					glrLinkedGuildsCount = glrLinkedGuildsCount + 1
				end
				if glrTempGuilds[guild] == nil then
					glrTempGuilds[guild] = guild
				end
			end
		end
		
	end
	
	sort(glrLinkedGuilds)
	sort(glrTempGuilds)
	
end